@@ -8,14 +8,10 @@
 # contained in the LICENSE file.
 
 """Unit tests for classes in models.py."""
+
 from typing import Any
 from unittest import TestCase
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
-
 import debusine.artifacts.models as data_models
 from debusine.assets.models import KeyPurpose
 
@@ -52,11 +48,9 @@ class DebusineHistoricalTaskRunTests(TestCase):
         data_models.DebusineHistoricalTaskRun(
             task_type=data_models.TaskTypes.WORKER, **kwargs
         )
-        with self.assertRaisesRegex(
-            ValueError, "is not a valid enumeration member"
-        ):
+        with self.assertRaisesRegex(ValueError, r"Input should be.*type=enum"):
             data_models.DebusineHistoricalTaskRun(
-                task_type="does-not-exist", **kwargs  # type: ignore[arg-type]
+                task_type="does-not-exist", **kwargs
             )
 
 
@@ -94,11 +88,9 @@ class DebusineTaskConfigurationTests(TestCase):
         data_models.DebusineTaskConfiguration(
             task_type=data_models.TaskTypes.WORKER, **kwargs
         )
-        with self.assertRaisesRegex(
-            ValueError, "is not a valid enumeration member"
-        ):
+        with self.assertRaisesRegex(ValueError, r"Input should be.*type=enum"):
             data_models.DebusineTaskConfiguration(
-                task_type="does-not-exist", **kwargs  # type: ignore[arg-type]
+                task_type="does-not-exist", **kwargs
             )
         with self.assertRaisesRegex(ValueError, "should be set"):
             data_models.DebusineTaskConfiguration(task_type=None, **kwargs)
@@ -226,7 +218,7 @@ class DebianSourcePackageTests(TestCase):
 
     def test_dsc_fields_missing(self) -> None:
         """Test checking that dsc_fields is present."""
-        error_msg = r"dsc_fields\s+field required \(type=value_error\.missing\)"
+        error_msg = r"dsc_fields\s+Field required.*type=missing"
 
         with self.assertRaisesRegex(Exception, error_msg):
             data_models.DebianSourcePackage(  # type: ignore[call-arg]
@@ -479,9 +471,7 @@ class DebianSystemTarballTest(TestCase):
             vendor="debian",
             codename="bookworm",
             components=["main"],
-            mirror=pydantic.parse_obj_as(
-                pydantic.AnyUrl, "https://deb.debian.org"
-            ),
+            mirror="https://deb.debian.org",
             variant=None,
             pkglist={},
             architecture="amd64",
@@ -501,9 +491,7 @@ class DebianSystemImageTest(TestCase):
             vendor="debian",
             codename="bookworm",
             components=["main"],
-            mirror=pydantic.parse_obj_as(
-                pydantic.AnyUrl, "https://deb.debian.org"
-            ),
+            mirror="https://deb.debian.org",
             variant=None,
             pkglist={},
             architecture="amd64",
@@ -559,9 +547,7 @@ class DebianAutopkgtestTest(TestCase):
             source_package=data_models.DebianAutopkgtestSource(
                 name="hello",
                 version="1.0-1",
-                url=pydantic.parse_obj_as(
-                    pydantic.AnyUrl, "https://deb.debian.org/pool/h/hello"
-                ),
+                url="https://deb.debian.org/pool/h/hello",
             ),
             architecture="amd64",
             distribution="bookworm",
@@ -613,7 +599,7 @@ class DebusinePromiseTests(TestCase):
             promise_work_request_id=1,
             promise_workflow_id=100,
             promise_category="debian:binary-package",
-            extra_field="extra_value",  # type: ignore
+            extra_field="extra_value",  # type: ignore[call-arg]
         )
 
         self.assertEqual(data.extra_field, "extra_value")  # type: ignore
@@ -628,7 +614,7 @@ class DebusinePromiseTests(TestCase):
                 promise_work_request_id=1,
                 promise_workflow_id=100,
                 promise_category="debian:binary-package",
-                promise_extra="not_allowed",  # type: ignore
+                promise_extra="not_allowed",  # type: ignore[call-arg]
             )
 
 
