@@ -21,9 +21,11 @@ from unittest.mock import MagicMock
 from urllib.parse import quote, urljoin
 
 import yaml
+from debian.deb822 import Dsc
 
 from debusine.artifacts import LocalArtifact
 from debusine.artifacts.models import ArtifactCategory
+from debusine.artifacts.playground import ArtifactPlayground
 from debusine.client import exceptions
 from debusine.client.commands.base import DebusineCommand
 from debusine.client.commands.tests.base import BaseCliTests
@@ -35,7 +37,6 @@ from debusine.client.models import (
     RelationType,
     RelationsResponse,
     RemoteArtifact,
-    model_to_json_serializable_dict,
 )
 from debusine.test.test_utils import (
     create_artifact_response,
@@ -47,14 +48,21 @@ class CliCreateArtifactTests(BaseCliTests):
     """Tests for the Cli functionality related to create artifacts."""
 
     def create_files_to_upload(self) -> list[Path]:
-        """Create three files to upload in the temp directory."""
-        return [
-            self.create_temporary_file(prefix="pkg_1.0.dsc", contents=b"test"),
-            self.create_temporary_file(
-                prefix="pkg_1.0.tar.gz", contents=b"test2"
-            ),
-            self.create_temporary_file(prefix="empty.txt", contents=b""),
-        ]
+        """Create three files to upload in a temporary directory."""
+        temp_directory = self.create_temporary_directory()
+        debian_tar_path = temp_directory / "pkg_1.0-1.debian.tar.gz"
+        debian_tar_path.write_bytes(b"test")
+        orig_tar_path = temp_directory / "pkg_1.0-1.orig.tar.gz"
+        orig_tar_path.write_bytes(b"test2")
+        dsc_path = temp_directory / "pkg_1.0-1.dsc"
+        ArtifactPlayground.write_deb822_file(
+            Dsc,
+            dsc_path,
+            [debian_tar_path, orig_tar_path],
+            source="pkg",
+            version="1.0-1",
+        )
+        return [dsc_path, debian_tar_path, orig_tar_path]
 
     def patch_create_artifact(self) -> MagicMock:
         """Patch Debusine.create_artifact."""
@@ -343,10 +351,10 @@ class CliCreateArtifactTests(BaseCliTests):
             scope=self.servers[self.default_server]["scope"],
             id=2,
             workspace=workspace,
-            files_to_upload=list(map(lambda p: p.name, files_to_upload)),
+            files_to_upload=[p.name for p in files_to_upload],
         )
 
-        stderr, stdout, _ = self.create_artifact_via_cli(
+        stderr, _, _ = self.create_artifact_via_cli(
             ArtifactCategory.WORK_REQUEST_DEBUG_LOGS,
             workspace=workspace,
             create_artifact_return_value=create_artifact_return_value,
@@ -386,11 +394,11 @@ class CliCreateArtifactTests(BaseCliTests):
             stderr,
             data_from_stdin=True,
         )
-        self.assertEqual(artifact_request.data, data)
+        self.assertEqual(artifact_request.data.model_dump(), data)
 
     def test_create_artifact_invalid_data_file_type(self) -> None:
         """Cli try to create an artifact, data file does not contain a dict."""
-        stderr, stdout, mocked_create_artifact = self.create_artifact_via_cli(
+        stderr, _, _ = self.create_artifact_via_cli(
             ArtifactCategory.SOURCE_PACKAGE,
             data=b"some-text",
             assert_system_exit_code=3,
@@ -402,7 +410,7 @@ class CliCreateArtifactTests(BaseCliTests):
 
     def test_create_artifact_invalid_yaml_in_data_file(self) -> None:
         """Cli try to create an artifact, data file contains invalid YAML."""
-        stderr, stdout, mocked_create_artifact = self.create_artifact_via_cli(
+        stderr, _, _ = self.create_artifact_via_cli(
             ArtifactCategory.SOURCE_PACKAGE,
             data=b'"',
             assert_system_exit_code=3,
@@ -414,7 +422,7 @@ class CliCreateArtifactTests(BaseCliTests):
     def test_create_artifact_cannot_read_data(self) -> None:
         """Cli try to create an artifact with a data file cannot be read."""
         data_file = "/tmp/debusine-test-file-does-not-exist"
-        stderr, stdout, mocked_create_artifact = self.create_artifact_via_cli(
+        stderr, _, _ = self.create_artifact_via_cli(
             ArtifactCategory.SOURCE_PACKAGE,
             data_file=Path(data_file),
             assert_system_exit_code=2,
@@ -434,25 +442,27 @@ class CliCreateArtifactTests(BaseCliTests):
             "type": "dpkg",
             "dsc_fields": {},
         }
-        files_to_upload: list[str] = []
+        files_to_upload = self.create_files_to_upload()
 
         create_artifact_return_value = create_artifact_response(
             base_url=self.get_base_url(self.default_server),
             scope=self.servers[self.default_server]["scope"],
             id=artifact_id,
             workspace=expected_workspace,
-            files_to_upload=files_to_upload,
+            files_to_upload=[path.name for path in files_to_upload],
         )
         artifact_category = ArtifactCategory.SOURCE_PACKAGE
-        (
-            stderr_actual,
-            stdout_actual,
-            mocked_create_artifact,
-        ) = self.create_artifact_via_cli(
-            artifact_category,
-            data=yaml.safe_dump(data).encode("utf-8"),
-            create_artifact_return_value=create_artifact_return_value,
-        )
+        with mock.patch.object(Debusine, "upload_files", autospec=True):
+            (
+                stderr_actual,
+                stdout_actual,
+                mocked_create_artifact,
+            ) = self.create_artifact_via_cli(
+                artifact_category,
+                data=yaml.safe_dump(data).encode("utf-8"),
+                create_artifact_return_value=create_artifact_return_value,
+                files_to_upload=[str(path) for path in files_to_upload],
+            )
 
         self.assert_artifact_is_created(
             mocked_create_artifact,
@@ -470,11 +480,7 @@ class CliCreateArtifactTests(BaseCliTests):
 
         create_artifact_side_effect = DebusineError(title=title_error)
 
-        (
-            stderr_actual,
-            stdout_actual,
-            mocked_create_artifact,
-        ) = self.create_artifact_via_cli(
+        stderr_actual, stdout_actual, _ = self.create_artifact_via_cli(
             ArtifactCategory.WORK_REQUEST_DEBUG_LOGS,
             workspace=workspace_name,
             create_artifact_side_effect=create_artifact_side_effect,
@@ -504,7 +510,7 @@ class CliCreateArtifactTests(BaseCliTests):
 
         paths_to_upload = [str(file_in_dir1), str(file_in_dir2)]
 
-        stderr, stdout, _ = self.create_artifact_via_cli(
+        stderr, _, _ = self.create_artifact_via_cli(
             ArtifactCategory.WORK_REQUEST_DEBUG_LOGS,
             workspace="some-workspace",
             files_to_upload=paths_to_upload,
@@ -520,9 +526,8 @@ class CliCreateArtifactTests(BaseCliTests):
 
     def test_create_artifact_invalid_category(self) -> None:
         """Cli try to create an artifact with a category that does not exist."""
-        stderr, stdout, mocked_create_artifact = self.create_artifact_via_cli(
-            "missing:does-not-exist",
-            assert_system_exit_code=2,
+        self.create_artifact_via_cli(
+            "missing:does-not-exist", assert_system_exit_code=2
         )
 
 
@@ -595,10 +600,10 @@ class CliImportDebianArtifactTests(BaseCliTests):
 
         def upload_artifact(
             _self: Debusine,
-            local_artifact: LocalArtifact[Any],  # noqa: U100
+            local_artifact: LocalArtifact[Any],  # noqa: ARG001, U100
             *,
-            workspace: str | None,  # noqa: U100
-            expire_at: dt.datetime | None,  # noqa: U100
+            workspace: str | None,
+            expire_at: dt.datetime | None,  # noqa: ARG001, U100
         ) -> RemoteArtifact:
             if workspace is None:  # pragma: no cover
                 workspace = "System"
@@ -688,7 +693,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
         uploaded_files = set()
         for args, kwargs in mocked_upload_artifact.call_args_list:
             self.assertEqual(len(args), 2)
-            debusine, local_artifact = args
+            _, local_artifact = args
             self.assertEqual(kwargs["workspace"], expected_workspace)
             self.assertEqual(kwargs["expire_at"], expected_expire_at)
             self.assertIsInstance(local_artifact, LocalArtifact)
@@ -837,7 +842,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
             "deb_fields": expected_deb_fields,
             "deb_control_files": ["control"],
         }
-        self.assertEqual(artifact.data, expected)
+        self.assertEqual(artifact.data.model_dump(), expected)
         self.assertEqual(set(artifact.files), {f"pkg_{version}_all.deb"})
 
     def assertUpload(
@@ -1027,7 +1032,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
         """Debusine.upload_artifact fails: cannot connect to the server."""
         files = self.create_dsc()
 
-        stderr, stdout, mocked_upload_artifact = self.import_debian_artifact(
+        stderr, _, _ = self.import_debian_artifact(
             workspace="test",
             upload=str(files[0]),
             upload_artifact_side_effect=exceptions.ClientConnectionError,
@@ -1038,7 +1043,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
     def test_import_debian_artifact_multiple_dsc(self) -> None:
         """import-debian-artifact fails when given two .dscs."""
         files = self.create_upload(source=True, binary=False, extra_dsc=True)
-        stderr, stdout, mocked_create_artifact = self.import_debian_artifact(
+        stderr, _, _ = self.import_debian_artifact(
             upload=files[0], assert_system_exit_code=3
         )
 
@@ -1066,7 +1071,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
                 """
             ).encode("utf-8")
         )
-        stderr, stdout, mocked_create_artifact = self.import_debian_artifact(
+        stderr, _, _ = self.import_debian_artifact(
             upload=changes, assert_system_exit_code=3
         )
 
@@ -1090,7 +1095,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
     def test_import_debian_artifact_missing_file(self) -> None:
         """import-debian-artifact fails when a named file is missing."""
         file = "/tmp/nonexistent.dsc"
-        stderr, stdout, mocked_create_artifact = self.import_debian_artifact(
+        self.import_debian_artifact(
             upload=file,
             exception_str=(
                 "[Errno 2] No such file or directory: '/tmp/nonexistent.dsc'"
@@ -1111,7 +1116,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
         title_error = f'Workspace "{workspace_name}" cannot be found'
         files = self.create_dsc()
 
-        stderr, stdout, mocked_upload_artifact = self.import_debian_artifact(
+        self.import_debian_artifact(
             workspace="test",
             upload=str(files[0]),
             upload_artifact_side_effect=DebusineError(title=title_error),
@@ -1148,8 +1153,8 @@ class CliShowArtifactTests(BaseCliTests):
         _, expected_output = self.capture_output(
             DebusineCommand._print_yaml,
             [
-                model_to_json_serializable_dict(
-                    debusine.return_value.artifact_get.return_value
+                debusine.return_value.artifact_get.return_value.model_dump(
+                    mode="json"
                 )
             ],
         )
@@ -1176,7 +1181,7 @@ class CliShowArtifactRelationsTests(BaseCliTests):
         self, *args: RelationResponse
     ) -> RelationsResponse:
         """Return a RelationsResponse containing args."""
-        return RelationsResponse(__root__=args)
+        return RelationsResponse(args)
 
     def test_show_artifact_relations(self) -> None:
         """Test show-artifact-relations shows all relations."""
@@ -1210,9 +1215,9 @@ class CliShowArtifactRelationsTests(BaseCliTests):
 
         self.assertEqual(stderr, "")
 
-        output = RelationsResponse(__root__=yaml.safe_load(stdout))
+        output = RelationsResponse(yaml.safe_load(stdout))
         expected = RelationsResponse(
-            __root__=[
+            [
                 RelationResponse(
                     artifact=11,
                     target=15,
@@ -1262,9 +1267,9 @@ class CliShowArtifactRelationsTests(BaseCliTests):
 
         self.assertEqual(stderr, "")
 
-        output = RelationsResponse(__root__=yaml.safe_load(stdout))
+        output = RelationsResponse(yaml.safe_load(stdout))
         expected = RelationsResponse(
-            __root__=[
+            [
                 RelationResponse(
                     artifact=8,
                     target=11,
