@@ -14,6 +14,7 @@ Debusine: Interacts with debusine server.
 
 Debusine fetches information, submit work requests and other operations.
 """
+
 import asyncio
 import datetime as dt
 import functools
@@ -32,6 +33,7 @@ from urllib.parse import quote, urlencode, urlparse, urlunparse
 
 import aiohttp
 import magic
+import pydantic
 import requests
 import tenacity
 from requests_toolbelt.downloadutils.stream import stream_response_to_file
@@ -76,8 +78,8 @@ from debusine.client.models import (
     WorkflowTemplateData,
     WorkflowTemplateDataNew,
     WorkspaceInheritanceChain,
-    model_to_json_serializable_dict,
 )
+from debusine.utils.urls import URLPath
 
 if TYPE_CHECKING:
     from debusine.client.task_configuration import (
@@ -137,22 +139,20 @@ def serialize_local_artifact(
     for artifact_path, file_request in artifact.remote_files.items():
         files[artifact_path] = file_request
 
-    artifact.validate_model()
+    artifact.model_validate(artifact)
 
-    serialized = model_to_json_serializable_dict(
-        ArtifactCreateRequest(
-            workspace=workspace,
-            category=artifact.category,
-            files=files,
-            data=(
-                artifact.data.dict()
-                if isinstance(artifact.data, ArtifactData)
-                else artifact.data
-            ),
-            work_request=work_request,
-            expire_at=expire_at,
-        )
-    )
+    serialized = ArtifactCreateRequest(
+        workspace=workspace,
+        category=artifact.category,
+        files=files,
+        data=(
+            artifact.data.model_dump()
+            if isinstance(artifact.data, ArtifactData)
+            else artifact.data
+        ),
+        work_request=work_request,
+        expire_at=expire_at,
+    ).model_dump(mode="json")
 
     # If the workspace was not specified: do not send it to the API.
     # The server will assign it.
@@ -200,10 +200,11 @@ class Debusine:
     def webui_url(self, path: str) -> str:
         """Generate a URL to the web UI given a full path."""
         parsed = urlparse(self.base_api_url)
-        new_path = (
-            os.path.dirname(parsed.path).rstrip("/") + "/" + path.lstrip('/')
-        )
-        parsed = parsed._replace(path=new_path)
+        parsed_path = URLPath(parsed.path)
+        if parsed_path and parsed_path[-1] == "api":
+            parsed_path.pop()
+        parsed_path += path
+        parsed = parsed._replace(path=str(parsed_path))
         return urlunparse(parsed)
 
     def work_request_iter(self) -> Iterable[WorkRequestResponse]:
@@ -272,8 +273,8 @@ class Debusine:
             path=self.work_request_completed_path(work_request_id),
             data={
                 "result": result,
-                "output_data": model_to_json_serializable_dict(
-                    output_data, exclude_unset=True
+                "output_data": output_data.model_dump(
+                    mode="json", exclude_unset=True
                 ),
             },
             expected_class=None,
@@ -288,7 +289,7 @@ class Debusine:
         :return: WorkRequest returned by the server.
         :raises: see _api_request method documentation.
         """
-        data = work_request.dict()
+        data = work_request.model_dump()
 
         # If workspace is None: the client does not want to specify
         # a workspace. Do not send it to the server: the server will
@@ -317,7 +318,7 @@ class Debusine:
         external_debsign_request: WorkRequestExternalDebsignRequest,
     ) -> WorkRequestResponse:
         """Provide external `debsign` data to a work request."""
-        data = external_debsign_request.dict()
+        data = external_debsign_request.model_dump()
 
         return self._debusine_http_client.post(
             f"/work-request/{work_request_id}/external-debsign/",
@@ -337,7 +338,7 @@ class Debusine:
         return self._debusine_http_client.post(
             f"/workflow-template/{quote(workspace)}/",
             WorkflowTemplateData,
-            data=workflow_template.dict(),
+            data=workflow_template.model_dump(),
             expected_statuses=[requests.codes.created],
         )
 
@@ -365,7 +366,7 @@ class Debusine:
         """Set information for a workflow template."""
         return self._debusine_http_client.patch(
             f"/workflow-template/{quote(workspace)}/{data.id}/",
-            data=model_to_json_serializable_dict(data),
+            data=data.model_dump(mode="json"),
             expected_class=WorkflowTemplateData,
         )
 
@@ -378,7 +379,7 @@ class Debusine:
         :return: Workflow WorkRequest returned by the server.
         :raises: see _api_request method documentation.
         """
-        data = create_workflow.dict()
+        data = create_workflow.model_dump()
 
         # If workspace is None: the client does not want to specify a
         # workspace. Do not send it to the server: the server will assign
@@ -529,7 +530,9 @@ class Debusine:
         return self._debusine_http_client.post(
             "/artifact-relation/",
             RelationResponse,
-            data=relation_request.dict(include={"artifact", "target", "type"}),
+            data=relation_request.model_dump(
+                include={"artifact", "target", "type"}
+            ),
             expected_statuses=[requests.codes.ok, requests.codes.created],
         )
 
@@ -578,13 +581,10 @@ class Debusine:
                 return False
         return total_size < 100 * 1024 * 1024
 
-    def _streaming_download(
-        self,
-        url: str,
-    ) -> requests.Response:
+    def _streaming_download(self, url: pydantic.AnyUrl) -> requests.Response:
         """Streaming download of url."""
         headers = {"Token": self.token}
-        response = requests.get(url, headers=headers, stream=True)
+        response = requests.get(str(url), headers=headers, stream=True)
         response.raise_for_status()
         return response
 
@@ -645,7 +645,7 @@ class Debusine:
         self,
         raw_response: io.IOBase,
         destination: Path,
-        url: str,
+        url: pydantic.AnyUrl,
     ) -> None:
         with (
             tempfile.TemporaryFile() as stderr_file,
@@ -723,14 +723,12 @@ class Debusine:
         return self._debusine_http_client.post(
             "/asset/",
             AssetResponse,
-            data=json.loads(
-                AssetCreateRequest(
-                    category=category,
-                    workspace=workspace,
-                    work_request=work_request,
-                    data=data,
-                ).json()
-            ),
+            data=AssetCreateRequest(
+                category=category,
+                workspace=workspace,
+                work_request=work_request,
+                data=data.model_dump(),
+            ).model_dump(),
             expected_statuses=[requests.codes.created],
         )
 
@@ -824,7 +822,7 @@ class Debusine:
             default_category=default_category,
         )
         return self._debusine_http_client.post(
-            "/lookup/single/", LookupSingleResponse, data=request.dict()
+            "/lookup/single/", LookupSingleResponse, data=request.model_dump()
         )
 
     @overload
@@ -863,13 +861,15 @@ class Debusine:
     ) -> LookupMultipleResponse[LookupSingleResponse]:
         """Look up multiple collection items."""
         request = LookupMultipleRequest(
-            lookup=lookup.dict()["__root__"],
+            lookup=lookup.model_dump(),
             work_request=work_request,
             expect_type=expect_type,
             default_category=default_category,
         )
         return self._debusine_http_client.post(
-            "/lookup/multiple/", LookupMultipleResponse, data=request.dict()
+            "/lookup/multiple/",
+            LookupMultipleResponse,
+            data=request.model_dump(),
         )
 
     def fetch_task_configuration_collection(
@@ -903,8 +903,8 @@ class Debusine:
             raise InvalidRepository("repository has no manifest")
 
         data = {
-            "collection": repo.manifest.collection.dict(),
-            "items": [i.item.dict() for i in repo.entries.values()],
+            "collection": repo.manifest.collection.model_dump(),
+            "items": [i.item.model_dump() for i in repo.entries.values()],
             "dry_run": dry_run,
         }
 
@@ -930,7 +930,7 @@ class Debusine:
         """Set a workspace inheritance chain."""
         return self._debusine_http_client.post(
             f"/workspace/{quote(workspace)}/inheritance/",
-            data=model_to_json_serializable_dict(chain),
+            data=chain.model_dump(mode="json"),
             expected_class=WorkspaceInheritanceChain,
         )
 
@@ -951,7 +951,7 @@ class Debusine:
         """Create a new collection."""
         return self._debusine_http_client.post(
             f"/collection/{quote(workspace)}/",
-            data=model_to_json_serializable_dict(data),
+            data=data.model_dump(mode="json"),
             expected_class=CollectionData,
             expected_statuses=[requests.codes.created],
         )
@@ -962,7 +962,7 @@ class Debusine:
         """Set information for a collection."""
         return self._debusine_http_client.patch(
             f"/collection/{quote(workspace)}/{data.id}/",
-            data=model_to_json_serializable_dict(data),
+            data=data.model_dump(mode="json"),
             expected_class=CollectionData,
         )
 
@@ -991,14 +991,9 @@ class Debusine:
         """Ask the server to emit a metric on behalf of this client."""
         self._debusine_http_client.post(
             "/open-metrics/emit/",
-            data=model_to_json_serializable_dict(
-                EmitMetricRequest(
-                    metric_type=metric_type,
-                    name=name,
-                    labels=labels,
-                    value=value,
-                )
-            ),
+            data=EmitMetricRequest(
+                metric_type=metric_type, name=name, labels=labels, value=value
+            ).model_dump(mode="json"),
             expected_class=None,
             expected_statuses=[requests.codes.no_content],
         )
@@ -1020,7 +1015,7 @@ class Debusine:
         last_completed_at: Path | None,
         msg_content: dict[str, Any],
     ) -> None:
-        on_work_request_completed = OnWorkRequestCompleted.parse_obj(
+        on_work_request_completed = OnWorkRequestCompleted.model_validate(
             msg_content
         )
 
@@ -1068,7 +1063,9 @@ class Debusine:
 
             self._logger.info("Requesting %s", url)
 
-            def connected(msg_content: dict[Any, Any]) -> None:  # noqa: U100
+            def connected(
+                msg_content: dict[Any, Any],  # noqa: ARG001, U100
+            ) -> None:
                 self._logger.info("Connected!")
 
             async with session.ws_connect(
