@@ -14,10 +14,7 @@ import datetime as dt
 import secrets
 from typing import Any, NotRequired, TypedDict
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic  # type: ignore
+import pydantic
 
 from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
 from debusine.client.models import (
@@ -39,11 +36,9 @@ from debusine.client.models import (
     RelationType,
     RelationsResponse,
     ServerObject,
-    StrMaxLength255,
     StrictBaseModel,
     WorkspaceInheritanceChain,
     WorkspaceInheritanceChainElement,
-    model_to_json_serializable_dict,
 )
 from debusine.test import TestCase
 from debusine.test.test_utils import (
@@ -60,13 +55,11 @@ class LinkTests(TestCase):
             (Link(href="foo"), True),
             (Link(href="foo", type="bar"), False),
         ):
-            with self.subTest(link=link.dict(exclude_unset=True)):
+            with self.subTest(link=link.model_dump(exclude_unset=True)):
                 self.assertEqual(link.can_be_str(), expected)
 
     def test_extra_arguments(self) -> None:
-        link = Link.parse_obj(
-            {"href": "foo", "rel": "bar", "title": "Baz", "type": "wibble"}
-        )
+        link = Link(href="foo", rel="bar", title="Baz", type="wibble")
         self.assertEqual(link.href, "foo")
         self.assertEqual(link.type, "wibble")
         self.assertFalse(hasattr(link, "rel"))
@@ -78,8 +71,7 @@ class ServerObjectTests(TestCase):
     def test_extra_json_fields(self) -> None:
         # Ensure client models ignore extra fields in serialized data, so that
         # new ones can be added without breaking old clients
-        serialized = {"extra": 1}
-        o = ServerObject.parse_obj(serialized)
+        o = ServerObject(extra=1)
         self.assertEqual(o.links, {})
         self.assertFalse(hasattr(o, "extra"))
 
@@ -110,10 +102,9 @@ class FileTests(TestCase):
         super().setUp()
         self.size = 10
         self.checksums = {
-            "sha256": pydantic.parse_obj_as(
-                StrMaxLength255,
+            "sha256": (
                 "bf2cb58a68f684d95a3b78ef8f661c"
-                "9a4e5b09e82cc8f9cc88cce90528caeb27",
+                "9a4e5b09e82cc8f9cc88cce90528caeb27"
             )
         }
 
@@ -131,7 +122,7 @@ class FileTests(TestCase):
         """Assert size cannot be negative."""
         with self.assertRaisesRegex(
             pydantic.ValidationError,
-            r"ensure this value is greater than or equal to 0",
+            r"Input should be greater than or equal to 0",
         ):
             FileRequest(size=-10, checksums=self.checksums, type="file")
 
@@ -142,19 +133,17 @@ class FileTests(TestCase):
     def test_file_type_only_file(self) -> None:
         """Assert file_type cannot be "link" (only "file")."""
         with self.assertRaisesRegex(
-            pydantic.ValidationError, r"permitted: 'file'.*given=link"
+            pydantic.ValidationError,
+            r"Input should be 'file'.*input_value='link'",
         ):
-            FileRequest(
-                size=self.size,
-                checksums=self.checksums,
-                type="link",  # type: ignore[arg-type]
-            )
+            FileRequest(size=self.size, checksums=self.checksums, type="link")
 
     def test_checksum_not_bigger_than_255(self) -> None:
         """Assert checksums cannot be longer than 255."""
-        checksums = {"sha256": StrMaxLength255("a" * 256)}
+        checksums = {"sha256": "a" * 256}
         with self.assertRaisesRegex(
-            pydantic.ValidationError, r"max_length.*255"
+            pydantic.ValidationError,
+            r"String should have at most 255 characters",
         ):
             FileRequest(size=self.size, checksums=checksums, type="file")
 
@@ -170,11 +159,7 @@ class ArtifactCreateRequestTests(TestCase):
         self.files = FilesRequestType(
             {
                 "README": FileRequest(
-                    size=10,
-                    checksums={
-                        "sha256": pydantic.parse_obj_as(StrMaxLength255, "aaaa")
-                    },
-                    type="file",
+                    size=10, checksums={"sha256": "aaaa"}, type="file"
                 )
             }
         )
@@ -231,7 +216,7 @@ class ArtifactResponseTests(TestCase):
             id=self.artifact_id,
             category="Testing",
             workspace="some workspace",
-            created_at=dt.datetime.now(dt.timezone.utc),
+            created_at=dt.datetime.now(dt.UTC),
             download_tar_gz_url="https://example.com/some/path/",
             files_to_upload=self.files_to_upload,
         )
@@ -275,11 +260,7 @@ class RelationCreateRequestTests(TestCase):
     def test_relation_type_raise_error(self) -> None:
         """Exception is raised if a non-valid relation type is set."""
         with self.assertRaises(pydantic.ValidationError):
-            RelationCreateRequest(
-                artifact=1,
-                target=2,
-                type="does-not-exist",  # type: ignore[arg-type]
-            )
+            RelationCreateRequest(artifact=1, target=2, type="does-not-exist")
 
 
 class RelationsResponseTests(TestCase):
@@ -287,7 +268,7 @@ class RelationsResponseTests(TestCase):
 
     def test_iter(self) -> None:
         """`__iter__` iterates over individual relation responses."""
-        response = RelationsResponse.parse_obj(
+        response = RelationsResponse.model_validate(
             [
                 {"id": i, "artifact": i + 1, "target": i, "type": "extends"}
                 for i in (1, 2)
@@ -311,7 +292,7 @@ class LookupMultipleResponseTests(TestCase):
     def test_iter(self) -> None:
         """`__iter__` iterates over individual lookup results."""
         response: LookupMultipleResponse[LookupSingleResponseArtifact] = (
-            LookupMultipleResponse.parse_obj(
+            LookupMultipleResponse.model_validate(
                 [
                     {"result_type": "a", "artifact": artifact}
                     for artifact in (1, 2)
@@ -354,27 +335,6 @@ class StrictBaseModelTests(TestCase):
             StrictBaseClass(does_not_exist=20)  # type: ignore[call-arg]
 
 
-class ModelToTest(pydantic.BaseModel):
-    """Simple model to be used in a test."""
-
-    id: int
-    created_at: dt.datetime
-
-
-class ModelToDictTests(TestCase):
-    """Tests for model_to_json_serializable_dict function."""
-
-    def test_model_to_json_serializable_dict(self) -> None:
-        """model_to_json_serializable_dict return datetime as str."""
-        obj_id = 4
-        created_at = dt.datetime.now(tz=dt.timezone.utc)
-
-        model = ModelToTest(id=obj_id, created_at=created_at)
-        expected = {"id": obj_id, "created_at": created_at.isoformat()}
-
-        self.assertEqual(model_to_json_serializable_dict(model), expected)
-
-
 class AssetResponseTest(TestCase):
     """Tests for the AssetResponse model."""
 
@@ -392,7 +352,7 @@ class AssetResponseTest(TestCase):
             },
             "work_request": 5,
         }
-        AssetResponse.parse_obj(body)
+        AssetResponse(**body)
 
     def test_parse_no_category(self) -> None:
         """Test parsing an asset without a category."""
@@ -404,7 +364,7 @@ class AssetResponseTest(TestCase):
             "work_request": 5,
         }
         with self.assertRaisesRegex(pydantic.ValidationError, r".*category.*"):
-            AssetResponse.parse_obj(body)
+            AssetResponse(**body)
 
 
 class AssetsResponsesTest(TestCase):
@@ -412,7 +372,7 @@ class AssetsResponsesTest(TestCase):
 
     def test_iter(self) -> None:
         """Test that AssetsResponses are iterable."""
-        responses = AssetsResponse.parse_obj(
+        responses = AssetsResponse(
             [
                 AssetResponse(
                     id=99,
@@ -423,8 +383,8 @@ class AssetsResponsesTest(TestCase):
                         fingerprint="ABC123",
                         public_key=base64.b64encode(b"PUBLIC KEY").decode(),
                         description="Description",
-                    ),
-                ).dict()
+                    ).model_dump(),
+                )
             ]
         )
         self.assertEqual(len(list(responses)), 1)
@@ -453,7 +413,9 @@ class EnrollPayloadTest(TestCase):
         ):
             with (
                 self.subTest(nonce=nonce),
-                self.assertRaisesRegex(ValueError, "Nonce is malformed"),
+                self.assertRaisesRegex(
+                    ValueError, r"\bnonce\s+String should match pattern"
+                ),
             ):
                 EnrollPayload(
                     nonce=nonce,
@@ -513,7 +475,9 @@ class EnrollPayloadTest(TestCase):
         ):
             with (
                 self.subTest(challenge=challenge),
-                self.assertRaisesRegex(ValueError, "Challenge is malformed"),
+                self.assertRaisesRegex(
+                    ValueError, r"\bchallenge\s+String should match pattern"
+                ),
             ):
                 EnrollPayload(
                     nonce="12345678",
@@ -578,7 +542,9 @@ class EnrollConfirmPayloadTest(TestCase):
         ):
             with (
                 self.subTest(token=token),
-                self.assertRaisesRegex(ValueError, "Token is malformed"),
+                self.assertRaisesRegex(
+                    ValueError, r"\btoken\s+String should match pattern"
+                ),
             ):
                 EnrollConfirmPayload(outcome=EnrollOutcome.CONFIRM, token=token)
 
