@@ -317,6 +317,17 @@ class WorkerManagerTests(TestCase):
             ["pool-001", "pool-003", "pool-004"],
         )
 
+    def test_get_or_create_scheduler(self) -> None:
+        """WorkerManager.get_or_create_scheduler returns a scheduler worker."""
+        worker = Worker.objects.get_or_create_scheduler()
+
+        self.assertEqual(worker.name, "scheduler")
+        self.assertIsNone(worker.token)
+        self.assertEqual(worker.worker_type, WorkerType.SCHEDULER)
+        self.assertIsNotNone(worker.pk)
+
+        self.assertEqual(Worker.objects.get_or_create_scheduler(), worker)
+
     def test_get_or_create_celery(self) -> None:
         """WorkerManager.get_or_create_celery returns a Celery worker."""
         worker = Worker.objects.get_or_create_celery()
@@ -415,18 +426,21 @@ class WorkerManagerTests(TestCase):
         self.assertGreaterEqual(idle_time, 0)
 
     def test_with_idle_time_completed_task(self) -> None:
-        """Compute idle_time with a running task."""
+        """Compute idle_time with a completed task."""
         worker = Worker.objects.create_with_fqdn(
             'worker-a', token=Token.objects.create(enabled=True)
         )
         worker.registered_at = timezone.now() - dt.timedelta(days=2)
         worker.instance_created_at = timezone.now() - dt.timedelta(days=1)
         worker.save()
-        work_request = self.playground.create_worker_task(
-            worker=worker, result=WorkRequest.Results.SUCCESS
-        )
-        work_request.completed_at = timezone.now() - dt.timedelta(hours=6)
-        work_request.save()
+        for hours_ago in (10, 6):
+            work_request = self.playground.create_worker_task(
+                worker=worker, result=WorkRequest.Results.SUCCESS
+            )
+            work_request.completed_at = timezone.now() - dt.timedelta(
+                hours=hours_ago
+            )
+            work_request.save()
 
         idle_time = (
             Worker.objects.with_idle_time().get(pk=worker.pk)
@@ -434,6 +448,28 @@ class WorkerManagerTests(TestCase):
         self.assertGreaterEqual(idle_time, 6 * 3600)
         self.assertLessEqual(idle_time, 7 * 3600)
 
+    def test_with_idle_time_old_completed_task(self) -> None:
+        """Compute idle_time with a completed task from a previous instance."""
+        worker = Worker.objects.create_with_fqdn(
+            'worker-a', token=Token.objects.create(enabled=True)
+        )
+        worker.registered_at = timezone.now() - dt.timedelta(days=7)
+        worker.instance_created_at = timezone.now() - dt.timedelta(seconds=5)
+        worker.save()
+        for hours_ago in (10, 6):
+            work_request = self.playground.create_worker_task(
+                worker=worker, result=WorkRequest.Results.SUCCESS
+            )
+            work_request.completed_at = timezone.now() - dt.timedelta(
+                hours=hours_ago
+            )
+            work_request.save()
+
+        idle_time = (
+            Worker.objects.with_idle_time().get(pk=worker.pk)
+        ).idle_time
+        self.assertLessEqual(idle_time, 60)
+
     def test_mark_all_disconnected(self) -> None:
         """WorkerManager.mark_all_disconnected() affects non-Celery workers."""
         worker_connected_external = self.playground.create_worker(
