@@ -9,8 +9,6 @@
 
 """Tests for the database playground."""
 
-import base64
-
 import django.test
 from django.conf import settings
 from django.contrib.auth.hashers import check_password
@@ -174,10 +172,10 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(pool.architectures, ["amd64"])
         self.assertEqual(pool.tags, ["a"])
         self.assertEqual(pool.provider_account, provider_account)
-        self.assertEqual(pool.specifications, specifications.dict())
+        self.assertEqual(pool.specifications, specifications.model_dump())
         self.assertFalse(pool.instance_wide)
         self.assertTrue(pool.ephemeral)
-        self.assertEqual(pool.limits, limits.dict())
+        self.assertEqual(pool.limits, limits.model_dump())
 
     def test_create_worker_pool_defaults(self) -> None:
         playground = Playground()
@@ -188,12 +186,11 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(pool.tags, [])
         self.assertIsNotNone(pool.provider_account, Asset)
         self.assertEqual(
-            pool.specifications,
-            DummyWorkerPoolSpecification().dict(),
+            pool.specifications, DummyWorkerPoolSpecification().model_dump()
         )
         self.assertTrue(pool.instance_wide)
         self.assertFalse(pool.ephemeral)
-        self.assertEqual(pool.limits, WorkerPoolLimits().dict())
+        self.assertEqual(pool.limits, WorkerPoolLimits().model_dump())
 
     def test_create_worker_pool_aws(self) -> None:
         playground = Playground()
@@ -204,7 +201,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         )
         self.assertEqual(
             pool.specifications,
-            AWSEC2WorkerPoolSpecification(launch_templates=[]).dict(),
+            AWSEC2WorkerPoolSpecification(launch_templates=[]).model_dump(),
         )
 
     def test_create_worker_pool_hetzner(self) -> None:
@@ -219,7 +216,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             HetznerCloudWorkerPoolSpecification(
                 server_type="cx22",
                 image_name="debian-13",
-            ).dict(),
+            ).model_dump(),
         )
 
     def test_create_worker_pool_unknown_provider(self) -> None:
@@ -549,7 +546,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         self.assertEqual(data_model.purpose, KeyPurpose.OPENPGP_UPLOAD)
         self.assertGreater(len(data_model.fingerprint), 8)
         self.assertTrue(
-            base64.b64decode(data_model.public_key).startswith(
+            data_model.public_key.startswith(
                 b"-----BEGIN PGP PUBLIC KEY BLOCK-----"
             )
         )
@@ -575,21 +572,21 @@ class PlaygroundTest(django.test.TestCase, TestCase):
         assert isinstance(data_model, SigningKeyData)
         self.assertEqual(data_model.purpose, KeyPurpose.UEFI)
         self.assertEqual(data_model.fingerprint, "ABCDEF")
-        self.assertEqual(base64.b64decode(data_model.public_key), b"PUBLIC KEY")
+        self.assertEqual(data_model.public_key, b"PUBLIC KEY")
         self.assertEqual(data_model.description, "Custom Asset")
 
     def test_create_cloud_provider_account_asset_defaults(self) -> None:
         playground = Playground()
         asset = playground.create_cloud_provider_account_asset()
         self.assertEqual(
-            asset.data, DummyProviderAccountData(name="test").dict()
+            asset.data, DummyProviderAccountData(name="test").model_dump()
         )
 
     def test_create_cloud_provider_account_asset_custom_data(self) -> None:
         playground = Playground()
         data = DummyProviderAccountData(name="test", secret="custom-secret")
         asset = playground.create_cloud_provider_account_asset(data=data)
-        self.assertEqual(asset.data, data.dict())
+        self.assertEqual(asset.data, data.model_dump())
 
     def test_create_cloud_provider_account_asset_aws(self) -> None:
         playground = Playground()
@@ -607,7 +604,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                     access_key_id="access-key",
                     secret_access_key="secret-key",
                 ),
-            ).dict(),
+            ).model_dump(),
         )
 
     def test_create_asset_usage_defaults(self) -> None:
@@ -685,7 +682,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 "components": ["main"],
                 'filename': 'test',
                 'mirror': 'https://deb.debian.org',
-                'pkglist': [],
+                'pkglist': {},
                 'variant': None,
                 'vendor': 'Debian',
                 "with_dev": True,
@@ -732,7 +729,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 "components": ["main"],
                 'filename': 'test',
                 'mirror': 'https://deb.debian.org',
-                'pkglist': [],
+                'pkglist': {},
                 "variant": "apt",
                 'vendor': 'Debian',
                 "with_dev": True,
@@ -786,7 +783,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
                 "components": ["main"],
                 'filename': 'test',
                 'mirror': 'https://deb.debian.org',
-                'pkglist': [],
+                'pkglist': {},
                 'variant': None,
                 'vendor': 'Debian',
                 "with_dev": True,
@@ -883,7 +880,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             task_data | {"task_configuration": task_config_collection.id},
         )
         self.assertEqual(
-            SbuildDynamicData.parse_obj(wr.dynamic_task_data),
+            SbuildDynamicData.model_validate(wr.dynamic_task_data),
             SbuildDynamicData(
                 environment_id=environment.pk,
                 input_source_artifact_id=source.pk,
@@ -1035,7 +1032,7 @@ class PlaygroundTest(django.test.TestCase, TestCase):
             },
         )
         self.assertEqual(
-            SbuildDynamicData.parse_obj(wr.dynamic_task_data),
+            SbuildDynamicData.model_validate(wr.dynamic_task_data),
             SbuildDynamicData(
                 environment_id=environment.pk,
                 input_source_artifact_id=source.pk,
