@@ -20,7 +20,7 @@ from collections.abc import Generator, Sequence
 from contextlib import ExitStack, contextmanager
 from dataclasses import dataclass, field
 from pathlib import Path, PurePath
-from typing import Generic, TypeVar
+from typing import override
 
 from debian.deb822 import Deb822, Packages, Sources
 from django.db import transaction
@@ -46,11 +46,9 @@ from debusine.tasks import DefaultDynamicData, TaskConfigError
 from debusine.tasks.models import BaseDynamicTaskData
 from debusine.utils import calculate_hash
 
-T = TypeVar("T", bound=Deb822)
-
 
 @dataclass
-class PlanAdd(Generic[T]):
+class PlanAdd[T: Deb822]:
     """A plan for adding a single item to a collection."""
 
     name: str
@@ -59,14 +57,14 @@ class PlanAdd(Generic[T]):
 
 
 @dataclass
-class PlanReplace(PlanAdd[T]):
+class PlanReplace[T: Deb822](PlanAdd[T]):
     """A plan for replacing a single item in a collection."""
 
     item: CollectionItem
 
 
 @dataclass
-class Plan(Generic[T]):
+class Plan[T: Deb822]:
     """A plan for updating items in a collection."""
 
     add: list[PlanAdd[T]] = field(default_factory=list)
@@ -110,12 +108,14 @@ class APTMirror(
         """The `debian:suite` collection this task is updating."""
         try:
             return Collection.objects.get(
-                name=self.data.collection, category=CollectionCategory.SUITE
+                name=self.data.collection,
+                category=CollectionCategory.SUITE,
+                workspace=self.workspace,
             )
         except Collection.DoesNotExist:
             raise TaskConfigError(
-                f"Collection '{self.data.collection}' with category "
-                f"'{CollectionCategory.SUITE}' not found"
+                f"Collection '{self.data.collection}' in '{self.workspace}' "
+                f"with category '{CollectionCategory.SUITE}' not found"
             )
 
     @functools.cached_property
@@ -213,7 +213,7 @@ class APTMirror(
 
         source = {
             "Types": "deb deb-src",
-            "URIs": self.data.url,
+            "URIs": str(self.data.url),
             "Suites": self.data.suite,
         }
         if self.data.components is not None:
@@ -767,6 +767,7 @@ class APTMirror(
             for remove in plan.remove:
                 self.collection_manager.remove_item(remove)
 
+    @override
     def _execute(self) -> bool:
         """Execute the task."""
         with advisory_lock(
@@ -801,6 +802,7 @@ class APTMirror(
                 self.update_indexes(self.plan_indexes(temp_path))
             return True
 
+    @override
     def get_label(self) -> str:
         """Return the task label."""
         return f"mirror {self.data.collection} from {self.data.url}"
