@@ -15,7 +15,7 @@ import tempfile
 from dataclasses import dataclass
 from functools import cache, cached_property
 from pathlib import Path, PurePath
-from typing import Any
+from typing import Any, override
 
 from debian.deb822 import Release
 from django.contrib.postgres.aggregates import StringAgg
@@ -169,7 +169,7 @@ class FunctionJoin(BaseTable):
     def as_sql(
         self,
         compiler: SQLCompiler,
-        connection: BaseDatabaseWrapper,  # noqa: U100
+        connection: BaseDatabaseWrapper,  # noqa: ARG002, U100
     ) -> tuple[str, list[str | int]]:
         """Generate an SQL fragment to be included in the current query."""
         alias_str = (
@@ -305,6 +305,7 @@ class GenerateSuiteIndexes(
         self.index_info: dict[PurePath, IndexInfo] = {}
         self.indexes: dict[PurePath, Artifact] = {}
 
+    @override
     def build_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> GenerateSuiteIndexesDynamicData:
@@ -315,6 +316,7 @@ class GenerateSuiteIndexes(
             ).id
         )
 
+    @override
     def get_input_artifacts_ids(self) -> list[int]:
         """Return the list of input artifact IDs used by this task."""
         # Our dynamic data includes a collection, but no artifacts.
@@ -926,6 +928,7 @@ class GenerateSuiteIndexes(
             .first()
         )
 
+    @override
     def _execute(self) -> bool:
         """Execute the task."""
         with advisory_lock(
@@ -971,6 +974,7 @@ class GenerateSuiteIndexes(
                 )
                 return True
 
+    @override
     def get_label(self) -> str:
         """Return the task label."""
         return (
