@@ -70,7 +70,7 @@ class RunServerTaskTests(TestCase):
     ) -> WorkRequest:
         """Create a work request and assign it to self.worker."""
         work_request = self.playground.create_server_task(
-            task_name=task_name, **kwargs
+            task_name=task_name, mark_pending=True, **kwargs
         )
         work_request.assign_worker(self.worker)
         return work_request
@@ -89,8 +89,9 @@ class RunServerTaskTests(TestCase):
 
     def test_not_pending(self) -> None:
         """The Celery task fails if the work request is not pending."""
-        work_request = self.create_assigned_work_request()
-        work_request.mark_completed(WorkRequest.Results.ERROR)
+        work_request = self.create_assigned_work_request(
+            result=WorkRequest.Results.ERROR
+        )
         with self.assertLogsContains(
             f"Work request {work_request.id} is in status completed, not "
             f"pending",
@@ -175,7 +176,10 @@ class RunServerTaskTests(TestCase):
     def test_external_task(self) -> None:
         """The Celery task logs but does not fail on an external task."""
         work_request = self.playground.create_worker_task(
-            task_name="noop", task_data={"result": True}, worker=self.worker
+            task_name="noop",
+            task_data={"result": True},
+            mark_pending=True,
+            worker=self.worker,
         )
         expected_message = "Cannot run on a Celery worker"
         with (
@@ -302,6 +306,7 @@ class RunServerTaskTransactionTests(TransactionTestCase):
         work_request = self.playground.create_server_task(
             task_name="sampletasknotmanagingtransactions",
             task_data={},
+            mark_pending=True,
         )
         work_request.assign_worker(worker)
         result = run_server_task.apply(args=(work_request.id,))
@@ -313,6 +318,7 @@ class RunServerTaskTransactionTests(TransactionTestCase):
         work_request = self.playground.create_server_task(
             task_name="sampletaskmanagingtransactions",
             task_data={},
+            mark_pending=True,
         )
         work_request.assign_worker(worker)
         result = run_server_task.apply(args=(work_request.id,))
@@ -381,10 +387,7 @@ class UpdateWorkflowTransactionTests(TransactionTestCase):
             self.assertTrue(work_request.workflows_need_update)
             unrelated = self.playground.create_worker_task()
 
-        with (
-            transaction.atomic(),
-            mock.patch("debusine.server.celery.set_transaction"),
-        ):
+        with transaction.atomic():
             result = update_workflows.apply()
             self.assertIsNone(result.get())
 
