@@ -10,7 +10,7 @@
 """Package publishing workflow."""
 
 import logging
-from typing import Any
+from typing import Any, override
 
 from debusine.artifacts.models import (
     ArtifactCategory,
@@ -28,7 +28,11 @@ from debusine.db.models import (
     WorkflowTemplate,
     Workspace,
 )
-from debusine.server.collections.lookup import lookup_multiple, lookup_single
+from debusine.server.collections.lookup import (
+    artifact_ids,
+    lookup_multiple,
+    lookup_single,
+)
 from debusine.server.tasks.models import (
     CopyCollectionItemsCopies,
     CopyCollectionItemsData,
@@ -36,17 +40,23 @@ from debusine.server.tasks.models import (
 from debusine.server.workflows import Workflow, workflow_utils
 from debusine.server.workflows.models import (
     PackagePublishWorkflowData,
+    PackagePublishWorkflowDynamicData,
     UpdateSuitesData,
     WorkRequestWorkflowData,
 )
-from debusine.tasks.models import BaseDynamicTaskData, LookupMultiple
+from debusine.tasks.models import (
+    InputArtifact,
+    InputArtifactMultiple,
+    InputArtifactSingle,
+    LookupMultiple,
+)
 from debusine.tasks.server import TaskDatabaseInterface
 
 logger = logging.getLogger(__name__)
 
 
 class PackagePublishWorkflow(
-    Workflow[PackagePublishWorkflowData, BaseDynamicTaskData]
+    Workflow[PackagePublishWorkflowData, PackagePublishWorkflowDynamicData]
 ):
     """Publish packages to a target suite."""
 
@@ -147,8 +157,8 @@ class PackagePublishWorkflow(
 
         copies.append(
             CopyCollectionItemsCopies(
-                source_items=LookupMultiple.parse_obj(
-                    [f"{source_artifact.id}@artifacts"]
+                source_items=LookupMultiple(
+                    (f"{source_artifact.id}@artifacts",)
                 ),
                 # Identify the target collection by ID to avoid strange
                 # behaviour if the lookup result changes between now and
@@ -222,8 +232,8 @@ class PackagePublishWorkflow(
 
         copies.append(
             CopyCollectionItemsCopies(
-                source_items=LookupMultiple.parse_obj(
-                    [f"{binary_artifact.id}@artifacts"]
+                source_items=LookupMultiple(
+                    (f"{binary_artifact.id}@artifacts",)
                 ),
                 # Identify the target collection by ID to avoid strange
                 # behaviour if the lookup result changes between now and
@@ -244,7 +254,7 @@ class PackagePublishWorkflow(
         target_workspace: Workspace,
     ) -> None:
         """Add copy items for build logs."""
-        if self.data.binary_artifacts.__root__:
+        if self.data.binary_artifacts:
             try:
                 source_build_logs_collection = self.lookup_singleton_collection(
                     CollectionCategory.PACKAGE_BUILD_LOGS
@@ -265,7 +275,7 @@ class PackagePublishWorkflow(
             ):
                 copies.append(
                     CopyCollectionItemsCopies(
-                        source_items=LookupMultiple.parse_obj(
+                        source_items=LookupMultiple.model_validate(
                             {
                                 "collection": source_build_logs_collection.id,
                                 "lookup__same_work_request": (
@@ -285,7 +295,7 @@ class PackagePublishWorkflow(
         target_workspace: Workspace,
     ) -> None:
         """Add copy items for task history."""
-        if self.data.binary_artifacts.__root__:
+        if self.data.binary_artifacts:
             try:
                 source_task_history_collection = (
                     self.lookup_singleton_collection(
@@ -311,7 +321,7 @@ class PackagePublishWorkflow(
             ):
                 copies.append(
                     CopyCollectionItemsCopies(
-                        source_items=LookupMultiple.parse_obj(
+                        source_items=LookupMultiple.model_validate(
                             {
                                 "collection": source_task_history_collection.id,
                                 "lookup__same_workflow": (
@@ -325,6 +335,7 @@ class PackagePublishWorkflow(
                     )
                 )
 
+    @override
     def populate(self) -> None:
         """Create work requests."""
         assert self.dynamic_data is not None
@@ -396,16 +407,17 @@ class PackagePublishWorkflow(
         )
         WorkRequest.objects.create_workflow(
             template=template,
-            data=UpdateSuitesData(only_suites=[self.target_suite.name]).dict(
-                exclude_unset=True
-            ),
+            data=UpdateSuitesData(
+                only_suites=[self.target_suite.name]
+            ).model_dump(exclude_unset=True),
         ).mark_pending()
         return True
 
+    @override
     def build_dynamic_data(
         self,
         task_database: TaskDatabaseInterface,  # noqa: U100
-    ) -> BaseDynamicTaskData:
+    ) -> PackagePublishWorkflowDynamicData:
         """
         Compute dynamic data for this workflow.
 
@@ -413,6 +425,9 @@ class PackagePublishWorkflow(
           by spaces
         :parameter_summary: source package names of ``binary_artifacts``
           separated by spaces → suite
+        :source_artifact_id: id of the source artifact
+        :binary_artifacts_ids: ids of the binary artifacts
+        :target_suite_id: collection id of the target_suite
         """
         binaries = lookup_multiple(
             self.data.binary_artifacts,
@@ -433,8 +448,55 @@ class PackagePublishWorkflow(
             )
         )
 
-        return BaseDynamicTaskData(
+        target_suite_id = lookup_single(
+            self.data.target_suite,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.workflow_root,
+            expect_type=LookupChildType.COLLECTION,
+        ).collection.id
+
+        try:
+            source_artifact_id = workflow_utils.source_package(self).id
+        except LookupError:
+            source_artifact_id = None
+
+        return PackagePublishWorkflowDynamicData(
             subject=source_package_names,
             parameter_summary=f"{source_package_names} → "
             f"{self.data.target_suite}",
+            source_artifact_id=source_artifact_id,
+            binary_artifacts_ids=artifact_ids(binaries),
+            target_suite_id=target_suite_id,
         )
+
+    @override
+    def get_input_artifacts(self) -> list[InputArtifact]:
+        """Return input artifacts."""
+        artifacts: list[InputArtifact] = []
+        if self.data.source_artifact is not None:
+            artifacts.append(
+                InputArtifactSingle(
+                    lookup=self.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=(
+                        self.dynamic_data.source_artifact_id
+                        if self.dynamic_data is not None
+                        else None
+                    ),
+                )
+            )
+
+        artifacts.append(
+            InputArtifactMultiple(
+                lookup=self.data.binary_artifacts,
+                label="binary_artifacts",
+                artifact_ids=(
+                    self.dynamic_data.binary_artifacts_ids
+                    if self.dynamic_data is not None
+                    else None
+                ),
+            )
+        )
+
+        return artifacts
