@@ -13,13 +13,6 @@ from collections.abc import Sequence
 from typing import Any
 from unittest.mock import call, patch
 
-from debusine.server.workflows.tests.helpers import WorkflowTestBase
-
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
-
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
@@ -38,7 +31,6 @@ from debusine.db.models import (
     WorkflowTemplate,
     default_workspace,
 )
-from debusine.server.tasks.models import PackageUploadTarget
 from debusine.server.workflows import (
     PackageUploadWorkflow,
     WorkflowValidationError,
@@ -46,11 +38,14 @@ from debusine.server.workflows import (
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
     PackageUploadWorkflowData,
+    PackageUploadWorkflowDynamicData,
     WorkRequestWorkflowData,
 )
+from debusine.server.workflows.tests.helpers import WorkflowTestBase
 from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
-    BaseDynamicTaskData,
+    InputArtifactMultiple,
+    InputArtifactSingle,
     LookupMultiple,
     SbuildData,
     SbuildInput,
@@ -77,19 +72,6 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         )
         return self.get_workflow(wr)
 
-    def create_source_package(self, *, name: str = "hello") -> Artifact:
-        """Create a minimal `debian:source-package` artifact."""
-        artifact, _ = self.playground.create_artifact(
-            category=ArtifactCategory.SOURCE_PACKAGE,
-            data={
-                "name": name,
-                "version": "1.0",
-                "type": "dpkg",
-                "dsc_fields": {},
-            },
-        )
-        return artifact
-
     def create_source_binary_artifacts(
         self, architectures: Sequence[str] | None = None
     ) -> tuple[Artifact, list[Artifact], Collection]:
@@ -100,7 +82,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
             "test", CollectionCategory.TEST
         )
 
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
         uploads = []
         for architecture in architectures:
             upload = self.playground.create_upload_artifacts(
@@ -137,7 +119,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         self,
     ) -> None:
         """Raise WorkflowValidationError: vendor and codename required."""
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
 
         w = self.create_package_upload_workflow(
             extra_task_data={"source_artifact": source.id},
@@ -170,8 +152,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
         self.assertEqual(w.data.source_artifact, source_artifact.id)
         self.assertEqual(
-            w.data.binary_artifacts,
-            LookupMultiple.parse_obj([binary_artifact.id]),
+            w.data.binary_artifacts, LookupMultiple((binary_artifact.id,))
         )
 
         self.assertEqual(w.data.target_distribution, target_distribution)
@@ -201,7 +182,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         -makesourcepackageupload
         -package_upload
         """
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
 
         with patch.object(
             PackageUploadWorkflow,
@@ -352,7 +333,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         assert parent.internal_collection is not None
 
         architectures = ("amd64", "i386")
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
         for architecture in architectures:
             sbuild = parent.create_child_worker(
                 task_name="sbuild",
@@ -767,7 +748,7 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_upload_delayed(self) -> None:
         """Upload to a delayed queue."""
-        source = self.create_source_package()
+        source = self.playground.create_minimal_source_package_artifact()
 
         wr = self.orchestrate(
             data={
@@ -794,8 +775,8 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
     def test_orchestrate_idempotent(self) -> None:
         """Calling orchestrate twice does not create new work requests."""
-        source, binaries, binaries_collection = (
-            self.create_source_binary_artifacts(architectures=("amd64", "i386"))
+        source, _, binaries_collection = self.create_source_binary_artifacts(
+            architectures=("amd64", "i386")
         )
 
         # Force a predictable default ordering.
@@ -828,16 +809,15 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         self.assertQuerySetEqual(wr.children.all(), children, ordered=False)
 
     def test_compute_dynamic_data_source_artifact(self) -> None:
-        source_artifact = self.create_source_package(name="hello")
+        source_artifact = (
+            self.playground.create_minimal_source_package_artifact()
+        )
 
         wr = self.playground.create_workflow(
             task_name="package_upload",
             task_data=PackageUploadWorkflowData(
                 source_artifact=source_artifact.id,
-                target=pydantic.parse_obj_as(
-                    PackageUploadTarget,
-                    "sftp://example.org",
-                ),
+                target="sftp://example.org",
                 vendor="debian",
                 codename="trixie",
             ),
@@ -846,8 +826,51 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="hello → sftp://example.org"
+            PackageUploadWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="hello → sftp://example.org",
+                source_artifact_id=source_artifact.id,
+                binary_artifacts_ids=[],
+            ),
+        )
+
+    def test_compute_dynamic_data_source_is_promise(self) -> None:
+        parent = self.playground.create_workflow()
+        assert parent.internal_collection is not None
+
+        signing = parent.create_child_worker(
+            task_name="noop",
+        )
+        self.playground.create_bare_data_item(
+            parent.internal_collection,
+            "source",
+            category=BareDataCategory.PROMISE,
+            data=DebusinePromise(
+                promise_work_request_id=signing.id,
+                promise_workflow_id=parent.id,
+                promise_category=ArtifactCategory.SOURCE_PACKAGE,
+                **{"source_package_name": "hello"},
+            ),
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                source_artifact="internal@collections/name:source",
+                target="sftp://example.org",
+                vendor="debian",
+                codename="trixie",
+            ),
+            parent=parent,
+        )
+
+        workflow = self.get_workflow(wr)
+        self.assertEqual(
+            workflow.build_dynamic_data(TaskDatabase(wr)),
+            PackageUploadWorkflowDynamicData(
+                parameter_summary="hello → sftp://example.org",
+                subject="hello",
+                binary_artifacts_ids=[],
             ),
         )
 
@@ -861,12 +884,11 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
         wr = self.playground.create_workflow(
             task_name="package_upload",
             task_data=PackageUploadWorkflowData(
-                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
-                source_artifact=None,
-                target=pydantic.parse_obj_as(
-                    PackageUploadTarget,
-                    "sftp://example.org",
+                binary_artifacts=LookupMultiple.model_validate(
+                    [binary_artifact.id]
                 ),
+                source_artifact=None,
+                target="sftp://example.org",
                 vendor="debian",
                 codename="trixie",
             ),
@@ -875,7 +897,84 @@ class PackageUploadWorkflowTests(WorkflowTestBase[PackageUploadWorkflow]):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="hello → sftp://example.org"
+            PackageUploadWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="hello → sftp://example.org",
+                source_artifact_id=None,
+                binary_artifacts_ids=[binary_artifact.id],
             ),
         )
+
+    def test_get_input_artifacts_source_artifact(self) -> None:
+        source_artifact = (
+            self.playground.create_minimal_source_package_artifact()
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                source_artifact=source_artifact.id,
+                target="sftp://example.org",
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        workflow.dynamic_data = workflow.build_dynamic_data(
+            TaskDatabase(workflow.work_request)
+        )
+
+        assert workflow.data.source_artifact is not None
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=source_artifact.id,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[],
+                ),
+            ],
+        )
+
+    def test_get_input_artifacts_binary_artifacts(self) -> None:
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello"
+            )
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="package_upload",
+            task_data=PackageUploadWorkflowData(
+                binary_artifacts=LookupMultiple.model_validate(
+                    [binary_artifact.id]
+                ),
+                source_artifact=None,
+                target="sftp://example.org",
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        workflow.dynamic_data = workflow.build_dynamic_data(
+            TaskDatabase(workflow.work_request)
+        )
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[binary_artifact.id],
+                )
+            ],
+        )
