@@ -23,11 +23,6 @@ from django.test import TransactionTestCase, override_settings
 from django.utils import timezone
 from nacl.public import PrivateKey
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
-
 from debusine.artifacts import (
     LocalArtifact,
     SigningInputArtifact,
@@ -49,7 +44,6 @@ from debusine.client.models import (
     FileResponse,
     FilesResponseType,
     RemoteArtifact,
-    StrMaxLength255,
 )
 from debusine.signing.db.models import Key
 from debusine.signing.gnupg import gpg_ephemeral_context
@@ -113,7 +107,7 @@ minimal_pecoff_image = (
 )
 
 
-example_url = pydantic.parse_obj_as(pydantic.AnyUrl, "http://example.org/")
+example_url = "http://example.org/"
 
 
 class SignTestMixin:
@@ -131,12 +125,7 @@ class SignTestMixin:
             {
                 name: FileResponse(
                     size=path.stat().st_size,
-                    checksums={
-                        "sha256": pydantic.parse_obj_as(
-                            StrMaxLength255,
-                            calculate_hash(path, "sha256").hex(),
-                        )
-                    },
+                    checksums={"sha256": calculate_hash(path, "sha256").hex()},
                     type="file",
                     url=example_url,
                 )
@@ -153,7 +142,7 @@ class SignTestMixin:
             workspace="System",
             category=artifact.category,
             created_at=timezone.now(),
-            data=artifact.data.dict(),
+            data=artifact.data.model_dump(),
             download_tar_gz_url=example_url,
             files_to_upload=[],
             files=self.make_files_response(artifact.files),
@@ -167,7 +156,11 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         """Test the analyze_worker() method."""
         self.mock_is_command_available({"sbsign": True})
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
         )
         metadata = task.analyze_worker()
         self.assertEqual(metadata["signing:sign:available:uefi"], True)
@@ -176,7 +169,11 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         """analyze_worker() handles sbsign not being available."""
         self.mock_is_command_available({"sbsign": False})
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
         )
         metadata = task.analyze_worker()
         self.assertEqual(metadata["signing:sign:available:uefi"], False)
@@ -184,7 +181,11 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
     def test_can_run_on_uefi(self) -> None:
         """can_run_on returns True for UEFI if sbsign is available."""
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
         )
         self.assertTrue(
             task.can_run_on(
@@ -202,7 +203,7 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             task_data={
                 "purpose": KeyPurpose.OPENPGP_UPLOAD,
                 "unsigned": [2],
-                "key": 1,
+                "key": "ABC123",
             }
         )
         self.assertTrue(
@@ -217,7 +218,11 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
     def test_can_run_on_mismatched_task_version(self) -> None:
         """can_run_on returns False for mismatched task versions."""
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
         )
         self.assertFalse(
             task.can_run_on(
@@ -232,7 +237,11 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
     def test_can_run_on_uefi_missing_tool(self) -> None:
         """can_run_on returns False for UEFI if sbsign is not available."""
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
         )
         self.assertFalse(
             task.can_run_on(
@@ -249,7 +258,7 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         task_db = FakeTaskDatabase(
             multiple_lookups={
                 # unsigned
-                (LookupMultiple.parse_obj([2]), None): [
+                (LookupMultiple((2,)), None): [
                     ArtifactInfo(
                         id=2,
                         category=ArtifactCategory.SIGNING_INPUT,
@@ -278,7 +287,7 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         task_db = FakeTaskDatabase(
             multiple_lookups={
                 # unsigned
-                (LookupMultiple.parse_obj([2]), None): [
+                (LookupMultiple((2,)), None): [
                     ArtifactInfo(
                         id=2,
                         category=ArtifactCategory.SIGNING_INPUT,
@@ -319,17 +328,25 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
     def test_fetch_input_missing_trusted_cert(self) -> None:
         """fetch_input requires trusted_certs to match configuration."""
         trusted_cert = "0" * 64
+        temp_directory = self.create_temporary_directory()
+        (input_file := temp_directory / "data").touch()
         signing_input = SigningInputArtifact.create(
-            [], self.create_temporary_directory(), trusted_certs=[trusted_cert]
+            [input_file], temp_directory, trusted_certs=[trusted_cert]
         )
 
         def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
+            artifact_id: int,
+            destination: Path,  # noqa: ARG001, U100
+            **kwargs: Any,
         ) -> ArtifactResponse:
             return self.make_artifact_response(signing_input, artifact_id)
 
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1},
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            },
             dynamic_task_data={"unsigned_ids": [2]},
         )
         task.work_request_id = 1
@@ -360,8 +377,10 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             private_key={},
             public_key=b"",
         )
+        temp_directory = self.create_temporary_directory()
+        (input_file := temp_directory / "data").touch()
         signing_input = SigningInputArtifact.create(
-            [], self.create_temporary_directory(), trusted_certs=[trusted_cert]
+            [input_file], temp_directory, trusted_certs=[trusted_cert]
         )
         signing_input_id = 2
         signing_input_response = self.make_artifact_response(
@@ -407,8 +426,10 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             private_key={},
         )
 
+        temp_directory = self.create_temporary_directory()
+        (input_file := temp_directory / "data").touch()
         signing_input = SigningInputArtifact.create(
-            [], self.create_temporary_directory()
+            [input_file], temp_directory
         )
         signing_input_id = 2
         signing_input_response = self.make_artifact_response(
@@ -450,8 +471,10 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
     def test_fetch_input_missing_key(self) -> None:
         """fetch_input requires the signing key to exist in the database."""
         fingerprint = "0" * 64
+        temp_directory = self.create_temporary_directory()
+        (input_file := temp_directory / "data").touch()
         signing_input = SigningInputArtifact.create(
-            [], self.create_temporary_directory()
+            [input_file], temp_directory
         )
         signing_input_id = 2
         signing_input_response = self.make_artifact_response(
@@ -499,10 +522,13 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             private_key={},
             public_key=b"",
         )
-        signing_inputs = [
-            SigningInputArtifact.create([], self.create_temporary_directory())
-            for _ in range(2)
-        ]
+        signing_inputs = []
+        for _ in range(2):
+            temp_directory = self.create_temporary_directory()
+            (input_file := temp_directory / "data").touch()
+            signing_inputs.append(
+                SigningInputArtifact.create([input_file], temp_directory)
+            )
         signing_input_ids = [2, 3]
         responses = {
             artifact_id: self.make_artifact_response(artifact, artifact_id)
@@ -513,7 +539,9 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         }
 
         def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
+            artifact_id: int,
+            destination: Path,  # noqa: ARG001, U100
+            **kwargs: Any,
         ) -> ArtifactResponse:
             return responses[artifact_id]
 
@@ -553,8 +581,10 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             private_key={},
             public_key=b"",
         )
+        temp_directory = self.create_temporary_directory()
+        (input_file := temp_directory / "data").touch()
         signing_input = SigningInputArtifact.create(
-            [], self.create_temporary_directory()
+            [input_file], temp_directory
         )
         signing_input_id = 2
         signing_input_response = self.make_artifact_response(
@@ -841,7 +871,7 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
         work_request_debug_logs_id = 6
 
         def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
+            artifact_id: int, destination: Path, **kwargs: Any
         ) -> ArtifactResponse:
             if artifact_id == signing_input_ids[0]:
                 shutil.copy(minimal_pecoff_path, destination)
@@ -1017,7 +1047,7 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
         signing_output_id = 3
 
         def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
+            artifact_id: int, destination: Path, **kwargs: Any
         ) -> ArtifactResponse:
             assert artifact_id == signing_input_id
             shutil.copy(changes_path, destination)
@@ -1145,6 +1175,10 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
     def test_label(self) -> None:
         """Test get_label."""
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
         )
         self.assertEqual(task.get_label(), "sign data for uefi")
