@@ -21,6 +21,7 @@ name of the class must be unique among all child classes.
 A child class must, at the very least, override the :py:meth:`BaseTask.execute`
 method.
 """
+
 import datetime as dt
 import inspect
 import logging
@@ -41,29 +42,24 @@ from typing import (
     AnyStr,
     BinaryIO,
     ClassVar,
-    Generic,
     IO,
     Literal,
     Self,
     TYPE_CHECKING,
     TextIO,
-    TypeVar,
     Union,
     assert_never,
     overload,
+    override,
 )
 
 if TYPE_CHECKING:
     from _typeshed import OpenBinaryModeWriting, OpenTextModeWriting
+
     from debusine.db.models import WorkRequest
 
 from debian.debian_support import Version
 
-try:
-    import pydantic.v1 as pydantic
-except ImportError:
-    import pydantic as pydantic  # type: ignore
-
 from debusine.artifacts import WorkRequestDebugLogs
 from debusine.artifacts.models import (
     ArtifactCategory,
@@ -105,6 +101,7 @@ from debusine.utils import extract_generic_type_arguments
 class TaskConfigError(Exception):
     """Exception raised when there is an issue with a task configuration."""
 
+    @override
     def __init__(
         self, message: str | None, original_exception: Exception | None = None
     ):
@@ -144,11 +141,7 @@ class TaskConfigError(Exception):
             return str(self.args[0])
 
 
-TD = TypeVar("TD", bound=BaseTaskData)
-DTD = TypeVar("DTD", bound=BaseDynamicTaskData)
-
-
-class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
+class BaseTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](metaclass=ABCMeta):
     """
     Base class for tasks.
 
@@ -272,8 +265,8 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         # extract IDs from dynamic_data for use by UI views
         self._source_artifacts_ids: list[int] = []
 
-        self._debug_log_files_directory: None | (
-            tempfile.TemporaryDirectory[str]
+        self._debug_log_files_directory: (
+            None | (tempfile.TemporaryDirectory[str])
         ) = None
 
     def set_work_request(self, work_request: "WorkRequest") -> None:
@@ -509,9 +502,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         :returns: the newly created dynamic task data
         """
 
-    def compute_dynamic_data(
-        self, task_database: TaskDatabaseInterface  # noqa: U100
-    ) -> DTD:
+    def compute_dynamic_data(self, task_database: TaskDatabaseInterface) -> DTD:
         """
         Compute dynamic data for this task.
 
@@ -685,7 +676,7 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
 
     def get_event_reactions(
         self,
-        event_name: Literal[  # noqa: U100
+        event_name: Literal[
             "on_creation",
             "on_unblock",
             "on_assignment",
@@ -705,7 +696,9 @@ class BaseTask(Generic[TD, DTD], metaclass=ABCMeta):
         return event_reactions
 
 
-class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
+class BaseExternalTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
+    BaseTask[TD, DTD], metaclass=ABCMeta
+):
     r"""
     A :py:class:`BaseTask` that runs on an external worker.
 
@@ -796,7 +789,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         self._source_artifacts_ids.append(artifact_id)
         return artifact_response
 
-    def fetch_input(self, destination: Path) -> bool:  # noqa: U100
+    def fetch_input(self, destination: Path) -> bool:  # noqa: ARG002, U100
         """
         Download artifacts needed by the task, update self.source_artifacts_ids.
 
@@ -809,7 +802,8 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         return True
 
     def configure_for_execution(
-        self, download_directory: Path  # noqa: U100
+        self,
+        download_directory: Path,  # noqa: ARG002, U100
     ) -> bool:
         """
         Configure task: set variables needed for the self._cmdline().
@@ -819,23 +813,29 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         return True
 
     def prepare_to_run(
-        self, download_directory: Path, execute_directory: Path  # noqa: U100
+        self,
+        download_directory: Path,  # noqa: ARG002, U100
+        execute_directory: Path,  # noqa: ARG002, U100
     ) -> None:
         """Prepare the execution environment to do the main work of the task."""
         # Do nothing by default.
 
     @abstractmethod
-    def run(self, execute_directory: Path) -> bool:  # noqa: U100
+    def run(self, execute_directory: Path) -> bool:  # noqa: ARG002, U100
         """Do the main work of the task."""
 
     def check_directory_for_consistency_errors(
-        self, build_directory: Path  # noqa: U100
+        self,
+        build_directory: Path,  # noqa: ARG002, U100
     ) -> list[str]:
         """Return list of errors after doing the main work of the task."""
         return []
 
     def upload_artifacts(
-        self, execute_directory: Path, *, execution_success: bool  # noqa: U100
+        self,
+        execute_directory: Path,
+        *,
+        execution_success: bool,  # noqa: U100
     ) -> None:
         """Upload the artifacts for the task."""
         # Do nothing by default.
@@ -844,6 +844,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
         """Clean up after running the task."""
         # Do nothing by default.
 
+    @override
     def _execute(self) -> bool:  # noqa: C901
         """
         Fetch the required input, execute the command and upload artifacts.
@@ -874,7 +875,7 @@ class BaseExternalTask(BaseTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta):
                     "Work request %s: %s", self.work_request_id, message
                 )
                 print(
-                    f"{dt.datetime.now(dt.timezone.utc).isoformat()} {message}",
+                    f"{dt.datetime.now(dt.UTC).isoformat()} {message}",
                     file=stages_log,
                 )
 
@@ -1074,12 +1075,11 @@ def get_environment(
     )
 
 
-TDE = TypeVar("TDE", bound=BaseTaskDataWithExecutor)
-DTDE = TypeVar("DTDE", bound=BaseDynamicTaskDataWithExecutor)
-
-
 # TODO: This should be an ABC.
-class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
+class BaseTaskWithExecutor[
+    TDE: BaseTaskDataWithExecutor,
+    DTDE: BaseDynamicTaskDataWithExecutor,
+](BaseExternalTask[TDE, DTDE]):
     r"""
     Base for tasks with executor capabilities.
 
@@ -1254,8 +1254,8 @@ class BaseTaskWithExecutor(BaseExternalTask[TDE, DTDE], Generic[TDE, DTDE]):
             self.executor_instance.stop()
 
 
-class RunCommandTask(
-    BaseExternalTask[TD, DTD], Generic[TD, DTD], metaclass=ABCMeta
+class RunCommandTask[TD: BaseTaskData, DTD: BaseDynamicTaskData](
+    BaseExternalTask[TD, DTD], metaclass=ABCMeta
 ):
     r"""
     A :py:class:`BaseTask` that can execute commands and upload artifacts.
@@ -1295,6 +1295,7 @@ class RunCommandTask(
     CMD_LOG_SEPARATOR = "--------------------"
     CMD_LOG_FILENAME = "cmd-output.log"
 
+    @override
     def run(self, execute_directory: Path) -> bool:
         """
         Run a single command via the executor.
@@ -1322,8 +1323,8 @@ class RunCommandTask(
 
     def task_succeeded(
         self,
-        returncode: int | None,  # noqa: U100
-        execute_directory: Path,  # noqa: U100
+        returncode: int | None,
+        execute_directory: Path,  # noqa: ARG002, U100
     ) -> bool:
         """
         Sub-tasks can evaluate if the task was a success or failure.
@@ -1583,13 +1584,10 @@ class RunCommandTask(
             pass
 
 
-TDER = TypeVar("TDER", bound=BaseTaskDataWithExtraRepositories)
-DTDER = TypeVar("DTDER", bound=BaseDynamicTaskDataWithExtraRepositories)
-
-
-class ExtraRepositoryMixin(
-    BaseExternalTask[TDER, DTDER], Generic[TDER, DTDER], metaclass=ABCMeta
-):
+class ExtraRepositoryMixin[
+    TDER: BaseTaskDataWithExtraRepositories,
+    DTDER: BaseDynamicTaskDataWithExtraRepositories,
+](BaseExternalTask[TDER, DTDER], metaclass=ABCMeta):
     """Methods for configuring external APT repositories."""
 
     data: TDER
@@ -1638,10 +1636,9 @@ class ExtraRepositoryMixin(
                     # certificate setup within environments, and we know that
                     # our own repositories are signed, so HTTP should be good
                     # enough to use from workers for now.
-                    url=pydantic.parse_obj_as(
-                        pydantic.AnyUrl,
+                    url=(
                         f"http://{self._get_archive_fqdn(task_database)}"
-                        f"/{suite.scope_name}/{suite.workspace_name}",
+                        f"/{suite.scope_name}/{suite.workspace_name}"
                     ),
                     suite=suite.name,
                     components=components,
@@ -1665,19 +1662,13 @@ class ExtraRepositoryMixin(
             for extra_repository in self.data.extra_repositories
         ]
 
-    def supports_deb822_sources(self, codename: str) -> bool:
-        """Determine if codename supports deb822 sources."""
-        return codename not in ("jessie", "trusty")
-
     def supports_inline_signed_by(self, codename: str) -> bool:
         """Determine if codename supports inline signatures."""
         # apt < 2.3.10 has no support for keys inline in Signed-By
         return codename not in (
-            "jessie",
             "stretch",
             "buster",
             "bullseye",
-            "trusty",
             "xenial",
             "bionic",
             "focal",
@@ -1692,9 +1683,8 @@ class ExtraRepositoryMixin(
         ``extra_repository_keys`` will be populated with keys to install
         into /etc/apt/keyrings, if needed.
 
-        If deb822 sources files are supported, ``extra_repository_sources``
-        will be populated with them; otherwise, ``extra_repository_lists``
-        will be populated with one-line sources.
+        ``extra_repository_sources`` will be populated with deb822 sources
+        files.
         """
         assert self.dynamic_data is not None
 
@@ -1710,46 +1700,27 @@ class ExtraRepositoryMixin(
                     self.extra_repository_keys.append(path)
                     signed_by = f"/etc/apt/keyrings/{path.name}"
 
-            # apt < 1.1 has no support for deb822 sources
-            if self.supports_deb822_sources(codename):
-                path = destination / f"extra_repository_{i}.sources"
-                path.write_text(
-                    extra_repo.as_deb822_source(signed_by_filename=signed_by)
-                )
-                self.extra_repository_sources.append(path)
-            else:
-                path = destination / f"extra_repository_{i}.list"
-                path.write_text(
-                    extra_repo.as_oneline_source(signed_by_filename=signed_by)
-                    + "\n"
-                )
-                self.extra_repository_lists.append(path)
-
-    def iter_oneline_sources(self) -> Generator[str]:
-        """
-        Generate apt one-line sources.list entries.
-
-        Use this for releases that don't supports_deb822_sources.
-        """
-        assert self.dynamic_data is not None
-
-        for i, extra_repository in enumerate(
-            self.dynamic_data.extra_repositories or []
-        ):
-            yield extra_repository.as_oneline_source(
-                signed_by_filename=f"/etc/apt/keyrings/extra_apt_key_{i}.asc"
+            path = destination / f"extra_repository_{i}.sources"
+            path.write_text(
+                extra_repo.as_deb822_source(signed_by_filename=signed_by)
             )
+            self.extra_repository_sources.append(path)
 
 
-class DefaultDynamicData(BaseTask[TD, BaseDynamicTaskData], metaclass=ABCMeta):
+class DefaultDynamicData[TD: BaseTaskData](
+    BaseTask[TD, BaseDynamicTaskData], metaclass=ABCMeta
+):
     """Base class for tasks that do not add to dynamic task data."""
 
+    @override
     def build_dynamic_data(
-        self, task_database: TaskDatabaseInterface  # noqa: U100
+        self,
+        task_database: TaskDatabaseInterface,  # noqa: U100
     ) -> BaseDynamicTaskData:
         """Return default dynamic data."""
         return BaseDynamicTaskData()
 
+    @override
     def get_input_artifacts_ids(self) -> list[int]:
         """Return the list of source artifact IDs used by this task."""
         return []
