@@ -17,7 +17,7 @@ import subprocess
 import tempfile
 from collections.abc import Generator
 from pathlib import Path, PurePath
-from typing import Any
+from typing import Any, override
 
 from debian.changelog import Changelog
 from debian.deb822 import Changes
@@ -32,7 +32,7 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianBinaryPackage,
-    get_architecture,
+    get_architectures,
     get_binary_package_name,
     get_binary_package_version,
 )
@@ -63,6 +63,7 @@ class AssembleSignedSource(
 
     TASK_VERSION = 1
 
+    @override
     def __init__(
         self,
         task_data: dict[str, Any],
@@ -83,10 +84,12 @@ class AssembleSignedSource(
         self._source_package_artifact: SourcePackage | None = None
         self._upload_artifact: Upload | None = None
 
+    @override
     def _cmdline(self) -> list[str]:
         """Unused abstract method from RunCommandTask."""
         raise NotImplementedError()
 
+    @override
     def can_run_on(self, worker_metadata: dict[str, Any]) -> bool:
         """Check if the specified worker can run the task."""
         if not super().can_run_on(worker_metadata):
@@ -95,6 +98,7 @@ class AssembleSignedSource(
         executor_available_key = f"executor:{self.backend}:available"
         return bool(worker_metadata.get(executor_available_key, False))
 
+    @override
     def build_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> AssembleSignedSourceDynamicData:
@@ -114,7 +118,9 @@ class AssembleSignedSource(
         assert isinstance(template.data, DebianBinaryPackage)
         package_name = get_binary_package_name(template.data)
         package_version = get_binary_package_version(template.data)
-        package_architecture = get_architecture(template.data)
+        architectures = get_architectures(template.data)
+        assert len(architectures) == 1
+        package_architectures = architectures.pop()
 
         return AssembleSignedSourceDynamicData(
             environment_id=self.get_environment(
@@ -128,9 +134,10 @@ class AssembleSignedSource(
             ).get_ids(),
             subject=package_name,
             parameter_summary=f"{package_name}_{package_version} "
-            f"({package_architecture})",
+            f"({package_architectures})",
         )
 
+    @override
     def get_input_artifacts_ids(self) -> list[int]:
         """Return the list of input artifact IDs used by this task."""
         if not self.dynamic_data:
@@ -165,10 +172,13 @@ class AssembleSignedSource(
 
         for relation in self.debusine.relation_list(artifact_id=artifact_id):
             if relation.type == relation_type:
-                artifact = self.debusine.artifact_get(relation.target)
+                artifact = self.debusine.artifact_get(
+                    relation.target, workspace=self.workspace_name
+                )
                 if artifact.category == target_category:
                     yield relation.target
 
+    @override
     def fetch_input(self, destination: Path) -> bool:
         """Download the required artifacts."""
         assert self.dynamic_data
@@ -227,8 +237,10 @@ class AssembleSignedSource(
 
         return True
 
+    @override
     def configure_for_execution(
-        self, download_directory: Path  # noqa: U100
+        self,
+        download_directory: Path,  # noqa: U100
     ) -> bool:
         """Configure task: ensure that the executor has dpkg-dev installed."""
         self._prepare_executor_instance()
@@ -440,6 +452,7 @@ class AssembleSignedSource(
         )
         self._upload_artifact = Upload.create(changes_file=local_changes_path)
 
+    @override
     def run(self, execute_directory: Path) -> bool:
         """Do the main assembly work."""
         assert self._template_deb_name is not None
@@ -475,8 +488,9 @@ class AssembleSignedSource(
 
         return True
 
+    @override
     def upload_artifacts(
-        self, execute_directory: Path, *, execution_success: bool  # noqa: U100
+        self, execute_directory: Path, *, execution_success: bool
     ) -> None:
         """Upload artifacts for the task."""
         assert self.work_request_id is not None
@@ -509,6 +523,7 @@ class AssembleSignedSource(
                 RelationType.EXTENDS,
             )
 
+    @override
     def cleanup(self) -> None:
         """Clean up after running the task."""
         if self._local_execute_directory is not None:
