@@ -23,7 +23,7 @@ from collections.abc import Collection
 from io import BytesIO
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import Any
+from typing import Any, override
 
 from debian.debian_support import Version
 
@@ -63,6 +63,7 @@ class Piuparts(
 
     CAPTURE_OUTPUT_FILENAME = "piuparts.txt"
 
+    @override
     def __init__(
         self,
         task_data: dict[str, Any],
@@ -84,6 +85,7 @@ class Piuparts(
         self._base_tar_data: DebianSystemTarball | None = None
         self._scripts_dir: Path | None = None
 
+    @override
     def build_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> PiupartsDynamicData:
@@ -167,6 +169,7 @@ class Piuparts(
             configuration_context=configuration_context,
         )
 
+    @override
     def get_input_artifacts_ids(self) -> list[int]:
         """Return the list of input artifact IDs used by this task."""
         if not self.dynamic_data:
@@ -177,6 +180,7 @@ class Piuparts(
             *self.dynamic_data.input_binary_artifacts_ids,
         ]
 
+    @override
     def fetch_input(self, destination: Path) -> bool:
         """Populate work directory with user-specified binary artifact(s)."""
         assert self.dynamic_data
@@ -185,6 +189,7 @@ class Piuparts(
             self.fetch_artifact(artifact_id, destination)
         return True
 
+    @override
     def configure_for_execution(self, download_directory: Path) -> bool:
         r"""
         Find the .deb files for piuparts.
@@ -291,7 +296,7 @@ class Piuparts(
         base_tgz_response = self.fetch_artifact(
             self.dynamic_data.base_tgz_id, base_tar_dir
         )
-        self._base_tar_data = DebianSystemTarball.parse_obj(
+        self._base_tar_data = DebianSystemTarball.model_validate(
             base_tgz_response.data
         )
         self._base_tar = base_tar_dir / self._base_tar_data.filename
@@ -424,43 +429,65 @@ class Piuparts(
 
     def _prepare_scripts(self, script_directory: Path) -> None:
         """Prepare scripts for piuparts."""
-        if not self.data.extra_repositories:
-            return
         if not self._base_tar_data:
             raise AssertionError(
                 "self._base_tar_data not set, call _prepare_base_tgz first."
             )
+
+        bad_permissions_script = (
+            script_directory / "pre_remove_50_find_bad_permissions"
+        )
+        bad_permissions_script.write_text(
+            "#!/bin/sh\n"
+            "echo 'Skipping to avoid https://bugs.debian.org/1125784'\n"
+        )
+        bad_permissions_script.chmod(0o755)
+
+        install_passwd_script = script_directory / "post_setup_00_passwd"
+        install_passwd_script.write_text(
+            "#!/bin/sh\n"
+            "echo 'Installing passwd for https://bugs.debian.org/1125789'\n"
+            "if [ ! -e /usr/sbin/useradd ]; then\n"
+            "    apt-get -y install passwd\n"
+            "fi\n"
+        )
+        install_passwd_script.chmod(0o755)
+
         codename = self._base_tar_data.codename
-        post_unpack_script = script_directory / "post_chroot_unpack_debusine"
-        with (
-            TemporaryDirectory(
-                prefix="debusine-piuparts-"
-            ) as extra_repositories_dir,
-            post_unpack_script.open("w") as f,
-        ):
-            extra_repositories_path = Path(extra_repositories_dir)
-            self.write_extra_repository_config(
-                codename=codename, destination=extra_repositories_path
+        if self.data.extra_repositories:
+            extra_repo_script = (
+                script_directory / "post_chroot_unpack_debusine_extra_repos"
             )
-
-            f.write("#!/bin/sh\nset -eux\n")
-            if self.extra_repository_keys:
-                f.write("mkdir -p /etc/apt/keyrings\n")
-            for key in self.extra_repository_keys:
-                f.write(
-                    f"printf %s {shlex.quote(key.read_text())}"
-                    f" > /etc/apt/keyrings/{key.name}\n"
-                )
-            for source in (
-                self.extra_repository_sources + self.extra_repository_lists
+            with (
+                TemporaryDirectory(
+                    prefix="debusine-piuparts-"
+                ) as extra_repositories_dir,
+                extra_repo_script.open("w") as f,
             ):
-                f.write(
-                    f"printf %s {shlex.quote(source.read_text())}"
-                    f" > /etc/apt/sources.list.d/{source.name}\n"
+                extra_repositories_path = Path(extra_repositories_dir)
+                self.write_extra_repository_config(
+                    codename=codename, destination=extra_repositories_path
                 )
-            f.write("echo DONE\n")
-        post_unpack_script.chmod(0o755)
 
+                f.write("#!/bin/sh\nset -eux\n")
+                if self.extra_repository_keys:
+                    f.write("mkdir -p /etc/apt/keyrings\n")
+                for key in self.extra_repository_keys:
+                    f.write(
+                        f"printf %s {shlex.quote(key.read_text())}"
+                        f" > /etc/apt/keyrings/{key.name}\n"
+                    )
+                for source in (
+                    self.extra_repository_sources + self.extra_repository_lists
+                ):
+                    f.write(
+                        f"printf %s {shlex.quote(source.read_text())}"
+                        f" > /etc/apt/sources.list.d/{source.name}\n"
+                    )
+                f.write("echo DONE\n")
+            extra_repo_script.chmod(0o755)
+
+    @override
     def _cmdline(self) -> list[str]:
         """Build full piuparts command line."""
         cmdline = ["piuparts"]
@@ -482,6 +509,8 @@ class Piuparts(
         # Use pre-generated tarballs (don't debootstrap on the fly)
         cmdline.append(f"--basetgz={self._base_tar}")
 
+        cmdline.append("--scriptsdir=/etc/piuparts/scripts")
+
         # Configure APT sources, etc.
         cmdline.append(f"--scriptsdir={self._scripts_dir}")
 
@@ -490,13 +519,15 @@ class Piuparts(
 
         return cmdline
 
+    @override
     @staticmethod
     def _cmdline_as_root() -> bool:
         """Piuparts must run as root."""
         return True
 
-    def execution_consistency_errors(
-        self, build_directory: Path  # noqa: U100
+    @override
+    def check_directory_for_consistency_errors(
+        self, build_directory: Path
     ) -> list[str]:
         """Return list of errors."""
         if build_directory / self.CAPTURE_OUTPUT_FILENAME not in set(
@@ -506,8 +537,12 @@ class Piuparts(
 
         return []
 
+    @override
     def upload_artifacts(
-        self, execute_directory: Path, *, execution_success: bool  # noqa: U100
+        self,
+        execute_directory: Path,
+        *,
+        execution_success: bool,  # noqa: U100
     ) -> None:
         """Upload a PiupartsArtifact with the files, data, and relationships."""
         assert self.debusine is not None
