@@ -8,6 +8,7 @@
 # contained in the LICENSE file.
 
 """Unit tests for MmDebstrap class."""
+
 import copy
 import shlex
 from pathlib import Path
@@ -215,7 +216,6 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
             "--format=tar",
             "--architectures=amd64",
             "--verbose",
-            "--hook-dir=/usr/share/mmdebstrap/hooks/maybe-jessie-or-older",
             (
                 '--customize-hook=cd "$1" && '
                 "find etc/apt/sources.list.d -type f -delete"
@@ -350,11 +350,11 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
         packages = self.patch_subprocess_run_pkglist()
 
         mirror = (
-            MmDebstrapData.parse_obj(self.SAMPLE_TASK_DATA)
+            MmDebstrapData.model_validate(self.SAMPLE_TASK_DATA)
             .bootstrap_repositories[0]
             .mirror
         )
-        bootstrap_options = MmDebstrapBootstrapOptions.parse_obj(
+        bootstrap_options = MmDebstrapBootstrapOptions.model_validate(
             self.SAMPLE_TASK_DATA["bootstrap_options"]
         )
 
@@ -387,62 +387,6 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
 
         debusine_mock.upload_artifact.assert_has_calls(calls)
 
-    def test_upload_artifacts_jessie(self) -> None:
-        """Test upload_artifacts() for a jessie tarball."""
-        directory = self.create_temporary_directory()
-
-        system_tarball = directory / MmDebstrap._OUTPUT_SYSTEM_FILE
-        system_tarball.write_bytes(b"Generated tarball")
-
-        # Debusine.upload_artifact is mocked to verify the call only
-        debusine_mock = self.mock_debusine()
-
-        os_release_data = self.write_os_release(
-            directory / self.task._OS_RELEASE_FILE, codename="jessie", version=8
-        )
-        (directory / self.task._TEST_SBIN_INIT_RETURN_CODE_FILE).write_text("0")
-
-        packages = self.patch_subprocess_run_pkglist()
-
-        mirror = (
-            MmDebstrapData.parse_obj(self.SAMPLE_TASK_DATA)
-            .bootstrap_repositories[0]
-            .mirror
-        )
-        bootstrap_options = MmDebstrapBootstrapOptions.parse_obj(
-            self.SAMPLE_TASK_DATA["bootstrap_options"]
-        )
-        self.task.data.bootstrap_repositories[0].suite = "jessie"
-
-        self.task.upload_artifacts(directory, execution_success=True)
-
-        calls = []
-
-        expected_system_artifact = DebianSystemTarballArtifact.create(
-            system_tarball,
-            data={
-                "variant": None,
-                "architecture": bootstrap_options.architecture,
-                "vendor": os_release_data["ID"],
-                "codename": "jessie",
-                "components": ["main", "contrib"],
-                "pkglist": packages,
-                "with_dev": True,
-                "with_init": True,
-                "mirror": mirror,
-            },
-        )
-
-        calls.append(
-            call(
-                expected_system_artifact,
-                workspace=self.task.workspace_name,
-                work_request=self.task.work_request_id,
-            )
-        )
-
-        debusine_mock.upload_artifact.assert_has_calls(calls)
-
     def test_upload_artifacts_sid(self) -> None:
         """Test upload_artifacts() for a sid tarball."""
         directory = self.create_temporary_directory()
@@ -463,11 +407,11 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
         packages = self.patch_subprocess_run_pkglist()
 
         mirror = (
-            MmDebstrapData.parse_obj(self.SAMPLE_TASK_DATA)
+            MmDebstrapData.model_validate(self.SAMPLE_TASK_DATA)
             .bootstrap_repositories[0]
             .mirror
         )
-        bootstrap_options = MmDebstrapBootstrapOptions.parse_obj(
+        bootstrap_options = MmDebstrapBootstrapOptions.model_validate(
             self.SAMPLE_TASK_DATA["bootstrap_options"]
         )
         self.task.data.bootstrap_repositories[0].suite = "unstable"
@@ -544,7 +488,7 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
 
         calls = []
 
-        bootstrap_options = MmDebstrapBootstrapOptions.parse_obj(
+        bootstrap_options = MmDebstrapBootstrapOptions.model_validate(
             self.SAMPLE_TASK_DATA["bootstrap_options"]
         )
         expected_system_artifact = DebianSystemTarballArtifact.create(
