@@ -8,12 +8,15 @@
 # contained in the LICENSE file.
 
 """Unit tests for SimpleSystemImageBuild class."""
+
 import shlex
 from copy import deepcopy
 from pathlib import Path
 from typing import Any, cast
 from unittest.mock import call, patch
 
+import pydantic
+
 from debusine.artifacts.local_artifact import DebianSystemImageArtifact
 from debusine.tasks import SimpleSystemImageBuild
 from debusine.tasks.models import (
@@ -148,7 +151,7 @@ class SimpleSystemImageBuildTests(
         self.configure_task(override={"bootstrap_options": bootstrap_options})
 
         self.assertEqual(
-            self.task.data.dict(exclude_unset=True),
+            self.task.data.model_dump(exclude_unset=True),
             {
                 'bootstrap_options': {
                     'architecture': 'amd64',
@@ -170,7 +173,9 @@ class SimpleSystemImageBuildTests(
                         ),
                         'components': ['main'],
                         'keyring': {
-                            'url': 'https://example.com/keyring.gpg',
+                            'url': pydantic.AnyUrl(
+                                'https://example.com/keyring.gpg'
+                            ),
                         },
                         'mirror': 'https://example.com',
                         'suite': 'bullseye',
