@@ -45,7 +45,6 @@ import debusine.tasks.models as task_models
 from debusine.artifacts.models import RuntimeStatistics, TaskTypes
 from debusine.client.debusine import Debusine
 from debusine.client.exceptions import TokenDisabledError
-from debusine.client.models import model_to_json_serializable_dict
 from debusine.tasks import (
     BaseTask,
     BaseTaskWithExecutor,
@@ -96,7 +95,9 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             {'General': {'api-url': self.api_url}}
         )
 
-        self.config = ConfigHandler(directories=[self.config_temp_directory])
+        self.config = ConfigHandler(
+            directories=[self.config_temp_directory.as_posix()]
+        )
         self.config['General']['log-file'] = '/dev/null'
 
         self.default_sigint_handler = signal.getsignal(signal.SIGINT)
@@ -417,8 +418,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             ),
         )
         log_message = (
-            'Could not register. Server HTTP status code: 0 '
-            'Body: Not available'
+            'Could not register. Server HTTP status code: 0 Body: Not available'
         )
         with (
             self.assertLogsContains(log_message),
@@ -525,7 +525,8 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         assert self.worker is not None
 
         async def process_message(
-            msg: aiohttp.WSMessage, client: Debusine  # noqa: U100
+            msg: aiohttp.WSMessage,  # noqa: ARG001, U100
+            client: Debusine,  # noqa: ARG001, U100
         ) -> bool:
             assert self.worker is not None
             await self.worker.close()
@@ -743,7 +744,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
                 build_architecture="amd64",
                 backend=BackendType.UNSHARE,
                 environment="debian/match:codename=sid",
-            ).dict()
+            ).model_dump()
         )
         with mock.patch(
             "debusine.tasks.executors.images.ImageCache.image_artifact",
@@ -1153,7 +1154,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         self.setup_worker()
         assert self.worker is not None
 
-        def sleep(delay: float) -> None:  # noqa: U100
+        def sleep(delay: float) -> None:  # noqa: ARG001, U100
             self.server_config.get_next_for_worker = (
                 ServerConfig.GetNextForWorkerResponse.VALID_WORK_REQUEST
             )
@@ -1252,7 +1253,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             workspace="TestWorkspace",
         )
         self.server_config.get_next_for_worker = json_response(
-            model_to_json_serializable_dict(next_work_request)
+            next_work_request.model_dump(mode="json")
         )
 
         await self.assert_execute_work_request_send_and_log_error(
@@ -1299,7 +1300,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             workspace="TestWorkspace",
         )
         self.server_config.get_next_for_worker = json_response(
-            model_to_json_serializable_dict(next_work_request)
+            next_work_request.model_dump(mode="json")
         )
 
         await self.assert_execute_work_request_send_and_log_error(
@@ -1328,7 +1329,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             workspace="TestWorkspace",
         )
         self.server_config.get_next_for_worker = json_response(
-            model_to_json_serializable_dict(next_work_request)
+            next_work_request.model_dump(mode="json")
         )
 
         await self.assert_execute_work_request_send_and_log_error(
@@ -1377,7 +1378,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             workspace="TestWorkspace",
         )
         self.server_config.get_next_for_worker = json_response(
-            model_to_json_serializable_dict(next_work_request)
+            next_work_request.model_dump(mode="json")
         )
 
         await self.receive_task_submit_result(
@@ -1409,7 +1410,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             workspace="TestWorkspace",
         )
         self.server_config.get_next_for_worker = json_response(
-            model_to_json_serializable_dict(next_work_request)
+            next_work_request.model_dump(mode="json")
         )
 
         await self.assert_execute_work_request_send_and_log_error(
@@ -1549,7 +1550,8 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         )
         with self.assertLogsContains(log_message, level=logging.DEBUG):
             message_processed = await self.worker._process_message(
-                msg, client  # type: ignore[arg-type]
+                msg,  # type: ignore[arg-type]
+                client,
             )
 
         self.assertFalse(message_processed)
@@ -1586,7 +1588,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
         config_temp_directory = self.create_temp_config_directory(
             {'General': {}}
         )
-        config = ConfigHandler(directories=[config_temp_directory])
+        config = ConfigHandler(directories=[config_temp_directory.as_posix()])
 
         log_message = (
             f'Missing required key "api-url" in General '
@@ -1604,7 +1606,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
     async def test_invalid_configuration_missing_general_section(self) -> None:
         """Worker() raises an exception: [General] section is not available."""
         config_temp_directory = self.create_temp_config_directory({})
-        config = ConfigHandler(directories=[config_temp_directory])
+        config = ConfigHandler(directories=[config_temp_directory.as_posix()])
 
         log_message = (
             f'Missing required section "General" in the configuration file '
@@ -1624,7 +1626,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             {"General": {"api-url": "http://example.org/"}}
         )
         config = ConfigHandler(
-            directories=[config_temp_directory], require_https=True
+            directories=[config_temp_directory.as_posix()], require_https=True
         )
 
         log_message = (
@@ -1645,7 +1647,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             {"General": {"api-url": "https://example.org/"}}
         )
         config = ConfigHandler(
-            directories=[config_temp_directory], require_https=True
+            directories=[config_temp_directory.as_posix()], require_https=True
         )
 
         self.setup_worker(config)
